/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;


/**
 * Provides definitions of the attribute change notifications sent by MBeans.
 * <P>
 * It's up to the MBean owning the attribute of interest to create and send
 * attribute change notifications when the attribute change occurs.
 * So the <CODE>NotificationBroadcaster</CODE> interface has to be implemented
 * by any MBean for which an attribute change is of interest.
 * <P>
 * Example:
 * If an MBean called <CODE>myMbean</CODE> needs to notify registered listeners
 * when its attribute:
 * <BLOCKQUOTE><CODE>
 *      String myString
 * </CODE></BLOCKQUOTE>
 * is modified, <CODE>myMbean</CODE> creates and emits the following notification:
 * <BLOCKQUOTE><CODE>
 * new AttributeChangeNotification(myMbean, sequenceNumber, timeStamp, msg,
 *                                 "myString", "String", oldValue, newValue);
 * </CODE></BLOCKQUOTE>
 */
public class AttributeChangeNotification extends Notification
{
	/* Serial version UID */
	private static final long serialVersionUID =535176054565814134L ;

	/**
	 * Notification type denoting that the observed MBean attribute value has
	 * changed. The value of this notification type is jmx.attribute.change.
	 **/
	public static final java.lang.String ATTRIBUTE_CHANGE = "jmx.attribute.change";

    /**
     * The MBean attribute name.
     */
    private String attributeName = null;

    /**
     * The MBean attribute type.
     */
    private String attributeType = null;

    /**
     * The MBean attribute old value.
     */
    private Object oldValue = null;

    /**
     * The MBean attribute new value.
     */
    private Object newValue = null;


    /**
     * Constructs an attribute change notification object. In addition to
	 * the information common to all notification, the caller must supply
	 * the name and type of the attribute, as well as its old and new values.
     *
     * @param source The notification producer, that is, the MBean
	 * 				the attribute belongs to.
	 *
     * @param sequenceNumber The notification sequence number within the source object.
     *
     * @param timeStamp The date at which the notification is being sent.
     *
     * @param msg A String containing the message of the notification.
     *
     * @param attributeName A String giving the name of the attribute.
     *
     * @param attributeType A String containing the type of the attribute.
     *
     * @param oldValue An object representing value of the attribute before the change.
     *
     * @param newValue An object representing value of the attribute after the change.
     */
	public AttributeChangeNotification( Object source,
                                   		long sequenceNumber,
										long timeStamp,
										String msg,
										String attributeName,
										String attributeType,
										Object oldValue,
										Object newValue)
	{
		super(ATTRIBUTE_CHANGE, source, sequenceNumber, timeStamp, msg);
		this.attributeName = attributeName;
		this.attributeType = attributeType;
		this.oldValue = oldValue;
		this.newValue = newValue;
	}

    /**
     * Gets the name of the attribute which has changed.
     *
     * @return A String containing the name of the attribute.
     */
	public String getAttributeName()
	{
		return attributeName;
	}

    /**
     * Gets the type of the attribute which has changed.
     *
     * @return A String containing the type of the attribute.
     */
	public String getAttributeType()
	{
		return attributeType;
	}

    /**
     * Gets the old value of the attribute which has changed.
     *
     * @return An Object containing the old value of the attribute.
     */
	public Object getOldValue()
	{
		return oldValue;
	}

    /**
     * Gets the new value of the attribute which has changed.
     *
     * @return An Object containing the new value of the attribute.
     */
	public Object getNewValue()
	{
		return newValue;
	}
}