/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;


/**
 * This class represents attributes used as arguments to relational constraints.
 * An <CODE>AttributeValueExp</CODE> may be used anywhere a
 * <CODE>ValueExp</CODE> is required.
 */
public class AttributeValueExp implements ValueExp
{
    /** The name of the attribute */
	private String attr = null;

	private String type = null;

	private Object retObj = null;

	private String className = null;

	private Object nobject = null;

	private transient MBeanServer server = null;

	private int qtype = -1;

	private AttributeValueExp v1 = null;

	private AttributeValueExp v2 = null;

	ObjectName mbeanName = null;

    //used in Query.classattr() method..
	boolean classattr = false;

	/**
	 * Basic Constructor.
	 */
	public AttributeValueExp()
	{
		classattr = true;
	}

	/**
     * Creates a new <CODE>AttributeValueExp</CODE> representing the specified
	 * object attribute, named attr.
	 *
     * @param attr creates AttributeValueExp with the specified attribute.
     */
	public AttributeValueExp(String attr)
	{
		this.attr = attr;
	}

	//this constructor may be replaced with NumericValueExp ,
	//called from Query.value(..)
	//confirm whether NumericValueExp is available...
	AttributeValueExp(String attr, Object nobject)
	{
		this.nobject = nobject;
	}

	//added for Query.attr(className, attr);
	AttributeValueExp(String className, String attr)
	{
		this.attr = attr;
		this.className = className;
	}

	//added for Query.plus(v1, v2)..
	AttributeValueExp(AttributeValueExp v1, AttributeValueExp v2, int qtype)
	{
		this.qtype = qtype;

		this.v1 = v1;
		this.v2 = v2;
	}

	/**
     * Applies the <CODE>AttributeValueExp</CODE> on an MBean.
     *
     * @param object The MBean objectname on which the
	 * 				<CODE>AttributeValueExp</CODE> will be applied.
     *
     * @return  The <CODE>ValueExp</CODE>.
     *
     * @exception BadAttributeValueExpException This ExpException is thrown
	 * 				when an invalid MBean attribute is passed to a query
	 * 				constructing method.
	 *
     * @exception InvalidApplicationException This exception is thrown when
	 * 				an attempt is made to apply either of the following:
	 * 				A subquery expression to an MBean
	 * 				A qualified attribute expression to an MBean of the wrong class
	 *
	 * @exception BadStringOperationException This exception is thrown when
	 * 				an invalid string operation is passed to a method for
	 * 				constructing a query.
	 *
	 * @exception BadBinaryOpValueExpException This exception is thrown when
	 * 				an invalid expression is passed to a method for
	 * 				constructing a query.
	 *
	 * @overrides apply in class ValueExp
	 */
	public ValueExp apply(ObjectName object)
					throws BadAttributeValueExpException,
						   InvalidApplicationException,
						   BadStringOperationException,
        		    	   BadBinaryOpValueExpException
	{
		this.mbeanName = object;

		if(v1 != null)
			v1.apply(object);

		if(v2 != null)
			v2.apply(object);

		retObj = null;

		type = null;

		//this condition(fails) useful when the Query.classattr case occurs...
		if(attr == null && nobject != null)
			return null;

		try
		{
			server.getObjectInstance(object);

		}
		catch(InstanceNotFoundException infe)
		{
			throw new InvalidApplicationException("MBean :: " + object +
													" :: not registered");
		}

		MBeanInfo info = null;

		try
		{
			info = server.getMBeanInfo(object);
		}
		catch(Exception e1)
		{
			return null;
		}

		if(className != null)
		{
			if(!className.equals(info.getClassName()))
				return null;
		}

		//probably then it would be Query.classattr case..
		if(attr == null && nobject == null && classattr)
		{
			retObj = info.getClassName();

			type = "java.lang.String";

			//attr = "";

			return null;
		}

		MBeanAttributeInfo[] attInfo = info.getAttributes();

		for(int i=0;i<attInfo.length;i++)
		{
			MBeanAttributeInfo att = attInfo[i];

			if(att.getName().equals(attr))
			{
				if(!att.isReadable())
					continue;
				type = att.getType();

				break;
			}
		}

		if(type == null)
		{
			return null;
		}

		try
		{
			retObj = server.getAttribute(object, attr);
		}
		catch(Exception e1)
		{
			return null;
		}

		if(retObj == null)
			return null;

		return getValueExp();
	}

	/**
	 * Returns a string representation of the name of the attribute.
	 *
	 * @return A string representation of the name of the attribute.
	 */
	public String getAttributeName()
	{
		return attr;
	}

	/**
	 * Sets the MBeanServer on which the query is to be accessed.
	 *
	 * @param s - The MBeanServer on which the query is to be accessed.
	 */
	public void setMBeanServer(MBeanServer server)
	{
		this.server = server;

		if(v1 != null)
			v1.setMBeanServer(server);
		if(v2 != null)
			v2.setMBeanServer(server);
	}

	/**
	 * Returns the string representing its value
	 *
	 * @overrides toString in class java.lang.Object
	 */
	public String toString()
	{
		if(attr != null || classattr)
			return retObj.toString();
		else if(nobject != null)
			return nobject.toString();

		if(qtype == Query.PLUS || qtype == Query.MINUS ||
			qtype == Query.TIMES || qtype == Query.DIV)
			return v1.toString() + v2.toString();

		return null;
	}

	//-------------------------- Protected methods --------------------------//

	protected Object getAttribute(Object mo)
	{
		return attr;
	}

	//---------------------- Package access methods -------------------------//

	Object getObject()
	{
		if(attr != null || classattr)
			return retObj;
		else if(nobject != null)
			return nobject;

		if(qtype == Query.PLUS || qtype == Query.MINUS ||
			qtype == Query.TIMES || qtype == Query.DIV)
		{
			return getCombinedObject(v1.getObject(), v2.getObject());
		}

		return null;
	}

	//--------------------------- Private methods ---------------------------//

	private ValueExp getValueExp()
	{
		ValueExp toRet = null;

		if(type.indexOf("String") != -1)
			toRet = Query.value((String)retObj);
		else if(type.indexOf("Integer") != -1 || type.indexOf("int") != -1)
			toRet = Query.value(((Integer)retObj).intValue());
		else if(type.indexOf("Long") != -1 || type.indexOf("long") != -1)
			toRet = Query.value(((Long)retObj).longValue());
		else if(type.indexOf("Boolean") != -1 || type.indexOf("boolean") != -1)
			toRet = Query.value(((Boolean)retObj).booleanValue());
		else if(type.indexOf("Float") != -1 || type.indexOf("float") != -1)
			toRet = Query.value(((Float)retObj).floatValue());
		else if(type.indexOf("Double") != -1 || type.indexOf("double") != -1)
			toRet = Query.value(((Double)retObj).doubleValue());
		else if(type.indexOf("Byte") != -1 || type.indexOf("byte") != -1)
			toRet = Query.value(((Byte)retObj).intValue());
		else if(type.indexOf("Character") != -1 || type.indexOf("char") != -1)
			toRet = Query.value( Character.getNumericValue(((Character)retObj).charValue()) );
		else if(type.indexOf("Short") != -1 || type.indexOf("short") != -1)
			toRet = Query.value(((Short)retObj).longValue());

		return toRet;
	}

	private Object getCombinedObject(Object o1, Object o2)
	{
		Object toRet = null;

		if((o1 instanceof Integer) && (o2 instanceof Long))
			o2 = new Integer(((Long)o2).intValue());

		if((o1 instanceof Long) && (o2 instanceof Integer))
			o1 = new Integer(((Long)o1).intValue());

		String o1Type = o1.getClass().getName();
		String o2Type = o2.getClass().getName();

		if(o1Type.indexOf("Integer") != -1 && o2Type.indexOf("Integer") != -1)
		{
			if(qtype == Query.PLUS)
			{
				toRet = new Integer( ((Integer)o1).intValue() +
										  ((Integer)o2).intValue() );
			}
			else if(qtype == Query.MINUS)
			{
				toRet = new Integer( ((Integer)o1).intValue() -
										  ((Integer)o2).intValue() );
			}
			else if(qtype == Query.TIMES)
			{
				toRet = new Integer( ((Integer)o1).intValue() *
										  ((Integer)o2).intValue() );
			}
			else if(qtype == Query.DIV)
			{
				toRet = new Integer( ((Integer)o1).intValue() /
										  ((Integer)o2).intValue() );
			}
		}
		else if(o1Type.indexOf("Long") != -1 && o2Type.indexOf("Long") != -1)
		{
			if(qtype == Query.PLUS)
			{
				toRet = new Long( ((Long)o1).longValue() +
									  ((Long)o2).longValue() );
			}
			else if(qtype == Query.MINUS)
			{
				toRet = new Long( ((Long)o1).longValue() -
									  ((Long)o2).longValue() );
			}
			else if(qtype == Query.TIMES)
			{
				toRet = new Long( ((Long)o1).longValue() *
									  ((Long)o2).longValue() );
			}
			else if(qtype == Query.DIV)
			{
				toRet = new Long( ((Long)o1).longValue() /
									  ((Long)o2).longValue() );
			}
		}
		else if(o1Type.indexOf("Float") != -1 && o2Type.indexOf("Float") != -1)
		{
			if(qtype == Query.PLUS)
			{
				toRet = new Float( ((Float)o1).floatValue() +
									  ((Float)o2).floatValue() );
			}
			else if(qtype == Query.MINUS)
			{
				toRet = new Float( ((Float)o1).floatValue() -
									  ((Float)o2).floatValue() );
			}
			else if(qtype == Query.TIMES)
			{
				toRet = new Float( ((Float)o1).floatValue() *
									  ((Float)o2).floatValue() );
			}
			else if(qtype == Query.DIV)
			{
				toRet = new Float( ((Float)o1).floatValue() /
									  ((Float)o2).floatValue() );
			}
		}
		else if(o1Type.indexOf("Double") != -1 && o2Type.indexOf("Double") != -1)
		{
			if(qtype == Query.PLUS)
			{
				toRet = new Double( ((Double)o1).doubleValue() +
									  ((Double)o2).doubleValue() );
			}
			else if(qtype == Query.MINUS)
			{
				toRet = new Double( ((Double)o1).doubleValue() -
									  ((Double)o2).doubleValue() );
			}
			else if(qtype == Query.TIMES)
			{
				toRet = new Double( ((Double)o1).doubleValue() *
									  ((Double)o2).doubleValue() );
			}
			else if(qtype == Query.DIV)
			{
				toRet = new Double( ((Double)o1).doubleValue() /
									  ((Double)o2).doubleValue() );
			}
		}
		else if(o1Type.indexOf("Byte") != -1 && o2Type.indexOf("Byte") != -1)
		{
			if(qtype == Query.PLUS)
			{
				toRet = new Byte( (byte)(((Byte)o1).byteValue() +
									  ((Byte)o2).byteValue()) );
			}
			else if(qtype == Query.MINUS)
			{
				toRet = new Byte( (byte)(((Byte)o1).byteValue() -
									  ((Byte)o2).byteValue()) );
			}
			else if(qtype == Query.TIMES)
			{
				toRet = new Byte( (byte)(((Byte)o1).byteValue() *
									  ((Byte)o2).byteValue()) );
			}
			else if(qtype == Query.DIV)
			{
				toRet = new Byte( (byte)(((Byte)o1).byteValue() /
									  ((Byte)o2).byteValue()) );
			}
		}
		else if(o1Type.indexOf("Short") != -1 && o2Type.indexOf("Short") != -1)
		{
			if(qtype == Query.PLUS)
			{
				toRet = new Short( (short)(((Short)o1).shortValue() +
									  ((Short)o2).shortValue()) );
			}
			else if(qtype == Query.MINUS)
			{
				toRet = new Short( (short)(((Short)o1).shortValue() -
									  ((Short)o2).shortValue()) );
			}
			else if(qtype == Query.TIMES)
			{
				toRet = new Short( (short)(((Short)o1).shortValue() *
									  ((Short)o2).shortValue()) );
			}
			else if(qtype == Query.DIV)
			{
				toRet = new Short( (short)(((Short)o1).shortValue() /
									  ((Short)o2).shortValue()) );
			}
		}

		return toRet;
	}
}