/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.lang.reflect.Constructor;
import java.io.Serializable;

/**
 * Describes a constructor exposed by an MBean.
 */
public class MBeanConstructorInfo extends MBeanFeatureInfo
								implements Serializable, Cloneable
{
    /**
     * The signature of the method, that is, the class names of the arguments.
     */
    private MBeanParameterInfo[] signature = null;

    /**
     * The constructor.
     */
    private transient java.lang.reflect.Constructor constructor = null;

	/* Serial version UID */
	private static final long serialVersionUID = 4433990064191844427L;

	/**
     * Constructs an <CODE>MBeanConstructorInfo</CODE> object.
     *
     * @param description A human readable description of the operation.
     * @param constructor The <CODE>java.lang.reflect.Constructor</CODE> object describing the MBean constructor.
	 */
	public MBeanConstructorInfo(String description, Constructor constructor)
	{
		super(constructor.getName(),description);
		this.constructor = constructor;
		this.signature = constructMBeanParameterInfo(constructor.getParameterTypes());
	}

	/**
     * Constructs an <CODE>MBeanConstructorInfo</CODE> object.
     *
     * @param name The name of the constructor.
     *
     * @param description A human readable description of the constructor.
     *
     * @param signature <CODE>MBeanParameterInfo</CODE> objects describing
	 * 				the parameters(arguments) of the constructor.
	 */
	public MBeanConstructorInfo(String name,
								String description,
								MBeanParameterInfo[] signature)
	{
		super(name, description);
		this.signature = signature;
	}

	/**
	* Returns the signature of the method, that is, information on
	* the operations arguments.
	*
	* @return A MBeanParameterInfo array is returned which describes
	* 				the parameters of the constructor
	*/
	public MBeanParameterInfo[] getSignature()
	{
		return signature;
	}

	/**
	 * Creates and returns a copy of this object.
	 *
	 * @return A duplicate copy of this object is created
	 */
	public Object clone()
	{
		MBeanParameterInfo[] clonedSig = new MBeanParameterInfo[signature.length];

		for(int i=0;i<signature.length;i++)
			clonedSig[i] = (MBeanParameterInfo)signature[i].clone();

		return new MBeanConstructorInfo(getName(), getDescription(), clonedSig);
	}

	/**
	 * Returns a human readable version of the MBeanConstructorInfo instance
	 *
	 * @return String format of MBeanConstructorInfo is returned
	 */
	public String toString()
	{
		return (super.toString() +
				"\n Signature = "+ getSignatureString());
	}

	//------------------------- Private methods ---------------------------//

	private String getSignatureString()
	{
		if(signature == null)
			return "null";

		String toRet = "";

		for (int i=0;i<signature.length;i++)
			toRet = (toRet + " [param"+ (i+1) + " = " + signature[i].toString() + "]\n" );

		return toRet;
	}

	private MBeanParameterInfo[] constructMBeanParameterInfo(Class[] params)
	{
		int size = params.length;
		MBeanParameterInfo[] mbParams = new MBeanParameterInfo[size];

		for(int i=0; i<size; i++)
		{
			mbParams[i] = new MBeanParameterInfo("",params[i].getName(),"");
		}

		return mbParams;
	}
}
