/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

//---------------------Importing the Java packages ---------------------------//
import java.util.Hashtable;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Properties;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * Represents the MBeanServer from the management point of view.
 * The MBeanServerDelegate MBean emits the MBeanServerNotifications
 * when a MBean is registered/deregistered in the MBean server.
 */
public class MBeanServerDelegate
		implements MBeanServerDelegateMBean, NotificationBroadcaster
{
	//----------------------------Variable Declaration------------------------//

	private String implProductName = "The XMOJO Project";
	private String implVendorName = "XMOJO (sponsored by AdventNet Inc.)";
	private String implVersion = "5.0";
	private String specName = "Java Management Extensions Instrumentation and Agent Specification";
	private String specVendorName = "Sun Microsystems Inc.";
	private String specVersion = "1.0, Final Release";

    /** The MBean server agent identification.*/
	private String serverId = "unknown";

	private Hashtable notifTable = new Hashtable();
	private Hashtable info = new Hashtable();

	/** Variable used to generate the serverId uniquely for each server **/
	private int genrandom = 1;

	private static int uniqueID = 1;

	private Properties properties = null;

	/**
	 * Create a MBeanServerDelegate object.
	 */
	public MBeanServerDelegate()
	{
		String host = "localhost";

		try
		{
			host = InetAddress.getLocalHost().getHostName();
			//Generate a serverId with host and random number.
			genrandom = uniqueID++;
			//genrandom = generateRandomInt(genrandom);
			serverId = host+"_"+genrandom;
		}
		catch(UnknownHostException UHE)
		{
			serverId = "unknown_"+uniqueID++;
		}

		initializeFromPropertiesFile();
	}

	/**
	 * Returns the JMX implementation name (the name of this product).
	 *
	 * @return The JMX implementation name is returned
	 */
	public String getImplementationName()
	{
		if(properties.getProperty("IMPL_NAME") != null)
			return properties.getProperty("IMPL_NAME");
		else
			return implProductName;
	}

	/**
	 * Returns the JMX implementation vendor (the vendor of this product)
	 *
	 * @return The JMX implementation vendor is returned
	 */
	public String getImplementationVendor()
	{
		if(properties.getProperty("VENDOR_NAME") != null)
			return properties.getProperty("VENDOR_NAME");
		else
			return implVendorName;
	}

	/**
	 * Returns the JMX implementation version (the version of this product).
	 *
	 * @return The JMX implementation version is returned
	 */
	public String getImplementationVersion()
	{
		if(properties.getProperty("VERSION") != null)
			return properties.getProperty("VERSION");
		else
			return implVersion;
	}

	/**
	 * Get the MBeanServer agent identification.
	 *
	 * @return MBeanServer ID is returned
	 */
	public java.lang.String getMBeanServerId()
	{
		return serverId;
	}


	/**
	 * Returns the full name of the JMX specification implemented by this
	 * product.
	 *
	 * @return The name of the JMX specification implemented by
	 * 				this product is returned
	 */
	public String getSpecificationName()
	{
		if(properties.getProperty("SPEC_NAME") != null)
			return properties.getProperty("SPEC_NAME");
		else
			return specName;
	}

	/**
	 * Returns the vendor of the JMX specification implemented by this product.
	 *
	 * @return This returns the vendor of the JMX specification
	 * 				implemented by this product
     */
	public String getSpecificationVendor()
	{
		if(properties.getProperty("SPEC_VENDOR") != null)
			return properties.getProperty("SPEC_VENDOR");
		else
			return specVendorName;
	}

	/**
	 * Returns the version of the JMX specification implemented by this product.
	 *
	 * @return The version of the JMX specification implemented by
	 * 				this product is returned
	 */
	public String getSpecificationVersion()
	{
		if(properties.getProperty("SPEC_VERSION") != null)
			return properties.getProperty("SPEC_VERSION");
		else
			return specVersion;
	}

	/**
	 * Enables a couple (listener,handback) for a registered MBean to be added.
	 * Specified by:
	 * addNotificationListener in interface NotificationBroadcaster
	 *
	 * param listener The listener object which will handles notifications
	 * 				emitted by the registered MBean.
	 *
	 * @param filter The filter object. If not specified, no filtering
	 * 				will be performed before
	 *
     * @param listener handling notifications.
     *
     * @param handback The context to be sent to the listener when a
     * 				notification is emitted.
     *
	 * @throws java.lang.IllegalArgumentException - Listener parameter is null.
	 */
	public void addNotificationListener(NotificationListener listener,
										NotificationFilter filter,
										Object handback)
										throws IllegalArgumentException
	{
		if(listener == null)
			throw new java.lang.IllegalArgumentException();

		ArrayList[] obj = (ArrayList[])notifTable.get(listener);
		if(obj == null)
		{
			obj = new ArrayList[2];
			obj[0] = new ArrayList();
			obj[1] = new ArrayList();
			obj[0].add(filter);
			obj[1].add(handback);

			notifTable.put(listener, obj);
		}
		else
		{
			boolean flag = false;
			Object[] handbacks = obj[1].toArray();
			for(int i=0; i<handbacks.length;i++)
			{
				if(handbacks[i].equals(handback))
				{
					flag = true;
					break;
				}
			}

			if(!flag)
			{
				obj[0].add(filter);
				obj[1].add(handback);
			}
		}
	}

	/**
	 * Enables a listener for an MBean to be removed. All couple (listener,
	 * handback) are removed.
	 *
	 * @param listener The listener object which will handles notifications
	 * 				emitted by the registered MBean.
	 *
	 * @throws ListenerNotFoundException The listener is not registered
	 * 				in the MBean.
	 */
	//Specified by:
	//removeNotificationListener in interface NotificationBroadcaster
	public void removeNotificationListener(NotificationListener listener)
								throws ListenerNotFoundException
	{
		notifTable.remove(listener);
	}

	/**
	 * Returns a NotificationInfo object contaning the name of the Java class
	 * of the notification and the notification types sent.
	 *
	 * @return This returns the array of MBeanNotificationInfo which
	 * 				contains the notification information
	 */
	public MBeanNotificationInfo[] getNotificationInfo()
	{
		MBeanNotificationInfo[] toRet = new MBeanNotificationInfo[info.size()];
		int i = 0;

		for(Enumeration e = info.keys();e.hasMoreElements();)
		{
			String key = (String)e.nextElement();
			List list = (List)info.get(key);
			toRet[i++] = new MBeanNotificationInfo((String[])list.toArray(),
											key, "Notification description");
		}

		return toRet;
	}

	/**
	 * Enables the MBean server to send a notification.
	 *
	 * @param notif The notification to send.
	 */
	public void sendNotification(Notification notif)
	{
		for(Enumeration e = notifTable.keys();e.hasMoreElements();)
		{
			NotificationListener nl = (NotificationListener)e.nextElement();

			ArrayList[] list = (ArrayList[])notifTable.get(nl);

			Object[] filters = list[0].toArray();
			Object[] handbacks = list[1].toArray();

			try
			{
				for(int i=0;i<filters.length;i++)
				{
					NotificationHandler hdlr = new NotificationHandler(
												(NotificationFilter)filters[i],
												nl,
												handbacks[i],
												notif,
												info);
					Thread th = new Thread(hdlr);
					th.start();
				}
			}
			catch(Exception ex)
			{
			}
		}
	}

	////////////////////////////// Package Methods /////////////////////////////

	class NotificationHandler implements Runnable
	{
		NotificationFilter nf;
		NotificationListener nl;
		Object handback;
		Notification notif;
		Hashtable info;

		NotificationHandler(NotificationFilter nf,
							NotificationListener nl,
							Object handback,
							Notification notif,
							Hashtable info)
		{
			this.nf = nf;
			this.nl = nl;
			this.handback = handback;
			this.notif = notif;
			this.info = info;
		}

		public void run()
		{
			if(nf != null)
			{
				if( nf.isNotificationEnabled(notif) )
				{
					nl.handleNotification(notif, handback);

					String name = notif.getClass().getName();
					List alist = (List)info.get(name);

					if(alist == null)
					{
						alist = new Vector();
						alist.add(notif.getType());

						info.put(name, alist);
					}
					else
						alist.add(notif.getType());
				}
		   }
		   else
		   		nl.handleNotification(notif,handback);
		}//End run()
	}

	////////////////////////////// Private Methods /////////////////////////////

	/**
	 * This method returns a random integer
	 */
	private int generateRandomInt(int prevId)
	{
		int random;
		random = ++prevId;
		random &= 0x7FFFFFFF;

		if (random > 0x3FFFFFFF)
		{
			random = random - 0x3FFFFFFF;
		}

		return random;
    }

    private void initializeFromPropertiesFile()
    {
		properties = new Properties();

		InputStream in = MBeanServerDelegate.class.getResourceAsStream(
												"/delegate.properties");

		try
		{
			properties.load(in);
		}
		catch(Exception e)
		{
			System.out.println("Could Not load entries from the Property File delegate.properties ");
		}
	}
}
