/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectInputStream.GetField;
import java.io.ObjectOutputStream;
import java.io.ObjectOutputStream.PutField;
import java.io.ObjectStreamField;
import java.io.IOException;

/**
 * This class is used for representing an object instance; that is,
 * an object name and class.  If the MBean is a Dynamic MBean the class name
 * should be retrieved from the <CODE>MBeanInfo</CODE> it provides.
 */
public class ObjectInstance implements Serializable
{
    /**
     * Object name.
     */
	private ObjectName name = null;

    /**
     * Class name.
     */
	private String className = null;

	private static final ObjectStreamField[] serialPersistentFields =
			{
				new ObjectStreamField("name", javax.management.ObjectName.class),
				new ObjectStreamField("className" ,java.lang.String.class)
			};

	//private static final long serialVersionUID = -4099952623687795850L;
	private static final long serialVersionUID = 0xc71a0acfad287b76L;


	/**
     * Allows an object instance to be created given a string representation of
     * an object name and the full class name, including the package name.
     *
     * @param objectName  A string representation of the object name.
     *
     * @param className  The full class name, including the package name,
	 * 				of the object instance. If the MBean is a Dynamic MBean
	 * 				the class name should be retrieved from the
	 * 				<CODE>MBeanInfo</CODE> it provides.
     *
     * @exception MalformedObjectNameException The string passed as a
	 * 				parameter does not have the right format.
	 */
	public ObjectInstance(String objectName, String className)
               							throws MalformedObjectNameException
	{
		if(objectName == null || className == null)
			throw new MalformedObjectNameException();

		this.name = new ObjectName(objectName);
		this.className = className;
	}

	/**
     * Allows an object instance to be created given an object name and
     * the full class name, including the package name.
     *
     * @param objectName  The object name.
     *
     * @param className  The full class name, including the package name,
	 * 				of the object instance. If the MBean is a Dynamic MBean
	 * 				the class name should be retrieved from the
	 * 				<CODE>MBeanInfo</CODE> it provides.
	 *
	 */
	public ObjectInstance(ObjectName objectName,String className)
	{
		this.name = objectName;
		this.className = className;
	}

	/**
	 * Compares the current object instance with another object instance.
	 *
	 * @param object The object instance that the current object instance is
	 * 				to be compared with.
	 *
	 * @return True if the two object instances are equal, otherwise false.
	 *
	 * @overrides equals in class java.lang.Object
	 */
	public boolean equals(Object object)
	{
		if( !(object instanceof ObjectInstance))
			return false;

		return name.toString().equals( ((ObjectInstance)object).getObjectName().toString() );
	}

	/**
	 * Returns the object name part.
	 *
	 * @return This reurns the object name
	 */
	public ObjectName getObjectName()
	{
		return name;
	}

	/**
	 * Returns the class part.
	 *
	 * @return This returns the class name
	 */
	public String getClassName()
	{
		return className;
	}

	/**
	 * Overrides toString() method
	 */
	public String toString()
	{
		return name.toString();
	}

	//------------------------- Private methods ---------------------------//

	private void readObject(ObjectInputStream objectinputstream)
			        			throws IOException, ClassNotFoundException
	{
		ObjectInputStream.GetField getfield = objectinputstream.readFields();

		try
		{
			name = (ObjectName)getfield.get("name", "");
			className=(String)getfield.get("className","");
		}
		catch(Exception exception)
		{
		}
	}

	private void writeObject(ObjectOutputStream objectoutputstream)
		          										throws IOException
	{
		ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
		putfield.put("name", name);
		putfield.put("className", className);

		objectoutputstream.writeFields();
	}
}