/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.modelmbean;

import java.lang.reflect.Method;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamField;
import java.io.IOException;

import javax.management.Descriptor;
import javax.management.DescriptorAccess;
import javax.management.MBeanAttributeInfo;
import javax.management.IntrospectionException;

/**
 * The ModelMBeanAttributeInfo object describes an attribute of the
 * ModelMBean. It is a subclass of MBeanAttributeInfo with the addition
 * of an associated Descriptor and an implementation of the DescriptorAccess
 * interface.
 * <p>
 * <p>
 * The fields in the descriptor are defined, but not limited to, the following:
 * <p>
 * <PRE>
 *	name           : attribute name
 *	descriptorType : must be "attribute"
 *	value          : current value for attribute
 *	default        : default value for attribute
 *	displayName    : name of attribute to be used in displays
 *	getMethod      : name of operation descriptor for get method
 *	setMethod      : name of operation descriptor for set method
 *	protocolMap    : object which implements the ProtocolMap interface: map of protocol names and protocol hints
 *	persistPolicy  : OnUpdate|OnTimer|NoMoreOftenThan|Always|Never
 *	persistPeriod  : seconds - frequency of persist cycle. Used when persistPolicy is"OnTimer" or "NoMoreOftenThan".
 *	currencyTimeLimit : how long value is valid, <0 never, =0 always, >0 seconds
 *	lastUpdatedTimeStamp : when value was set
 *	iterable       : T - object value supports Iterable interface, F - does not support Iterable interface
 *	visibility     : 1-4 where 1: always visible 4: rarely visible
 *	presentationString : xml formatted string to allow presentation of data
 * </PRE>
 * <p>The default descriptor contains the name and descriptorType fields.
 **/
public class ModelMBeanAttributeInfo extends MBeanAttributeInfo
									implements DescriptorAccess, Cloneable
{
	String [] fieldNames={"name","descriptorType"};
	String [] fieldValues={getName(),new String("attribute")};

	Descriptor descriptor = new DescriptorSupport(fieldNames,fieldValues);

	private static final long serialVersionUID = 0x62814af8f0446c19L;

	private static final ObjectStreamField[] serialPersistentFields =
		{
			new ObjectStreamField("attrDescriptor", DescriptorSupport.class),
			new ObjectStreamField("currClass",java.lang.String.class)

        };


	/**
	 * Constructs a ModelMBeanAttributeInfo object with a default descriptor.
	 *
	 * @param name The name of the attribute.
	 *
	 * @param description A human readable description of the attribute. Optional.
	 *
	 * @param getter The method used for reading the attribute value.
	 * 				May be null if the property is write-only.
	 *
	 * @param setter The method used for writing the attribute value.
	 *          	May be null if the attribute is read-only.
	 *
	 * @exception IntrospectionException There is a consistency problem in
	 * 				the definition of this attribute.
	 */
	public ModelMBeanAttributeInfo(String name,
									String description,
									Method getter,
									Method setter)
							throws IntrospectionException
	{
		super(name, description, getter, setter);
	}

	/**
	 * Constructs a ModelMBeanAttributeInfo object.
	 *
	 * @param name The name of the attribute.
	 *
	 * @param description A human readable description of the attribute. Optional.
	 *
	 * @param getter The method used for reading the attribute value.
	 *          	May be null if the property is write-only.
	 *
	 * @param setter The method used for writing the attribute value.
	 *          	May be null if the attribute is read-only.
	 *
	 * @param descriptor An instance of Descriptor containing the appropriate
	 * 				metadata for this instance of the Attribute. If it is null
	 * 				or invalid then a default desriptor will be created.
	 *
	 * @exception IntrospectionException There is a consistency problem in
	 * 				the definition of this attribute.
	 */
	public ModelMBeanAttributeInfo(String name,
									String description,
									Method getter,
									Method setter,
									Descriptor descriptor)
							throws IntrospectionException
	{
		super(name, description, getter, setter);
		this.descriptor = descriptor;
	}

	/**
	 * Constructs a ModelMBeanAttributeInfo object with a default descriptor.
	 *
	 * @param name The name of the attribute
	 *
	 * @param type The type or class name of the attribute
	 *
	 * @param description A human readable description of the attribute.
	 *
	 * @param isReadable True if the attribute has a getter method, false otherwise.
	 *
	 * @param isWritable True if the attribute has a setter method, false otherwise.
	 *
	 * @param isIs True if the attribute has an "is" getter, false otherwise.
	 */
	public ModelMBeanAttributeInfo(String name,
								    String type,
									String description,
									boolean isReadable,
									boolean isWritable,
									boolean isIs)
	{
		super(name, type, description, isReadable, isWritable, isIs);
	}

	/**
	 * Constructs a ModelMBeanAttributeInfo object with a default descriptor.
	 *
	 * @param name The name of the attribute
	 *
	 * @param type The type or class name of the attribute
	 *
	 * @param description A human readable description of the attribute.
	 *
	 * @param isReadable True if the attribute has a getter method, false otherwise.
	 *
	 * @param isWritable True if the attribute has a setter method, false otherwise.
	 *
	 * @param isIs True if the attribute has an "is" getter, false otherwise.
	 *
	 * @param descriptor An instance of Descriptor containing the appropriate
	 * 				metadata for this instance of the Attribute. If it is null
	 * 				or invalid then a default desriptor will be created.
	 */
	public ModelMBeanAttributeInfo(String name,
								    String type,
									String description,
									boolean isReadable,
									boolean isWritable,
									boolean isIs,
									Descriptor descriptor)
	{
		super(name, type, description, isReadable, isWritable, isIs);
		this.descriptor = descriptor;
	}

	/**
	 * Constructs a new ModelMBeanAttributeInfo object from this
	 * ModelMBeanAttributeInfo Object. A default descriptor will be created.
	 *
	 * @param inInfo the ModelMBeanAttributeInfo to be duplicated
	 */
	public ModelMBeanAttributeInfo(ModelMBeanAttributeInfo inInfo)
	{
		super(inInfo.getName(), inInfo.getType(), inInfo.getDescription(),
			  inInfo.isReadable(), inInfo.isWritable(), inInfo.isIs());

		if(inInfo.getDescriptor() != null)
			this.descriptor = (Descriptor)inInfo.getDescriptor().clone();
	}

	/**
	 * Creates and returns a new ModelMBeanAttributeInfo which is a
	 * duplicate of this ModelMBeanAttributeInfo.
	 *
	 * @exception RuntimeOperationsException for illegal value for field Names
	 * 				or field Values. If the descriptor construction fails for
	 * 				any reason, this exception will be thrown.
	 */
	public Object clone()
	{
		Descriptor clonedDescr = null;

		if(descriptor != null)
			clonedDescr = (Descriptor) descriptor.clone();

		try
		{
			return new ModelMBeanAttributeInfo(getName(), getType(),
								getDescription(), isReadable(), isWritable(),
								isIs(), clonedDescr);
		}
		catch(Exception e)
		{
			return null;
		}
	}

	/**
	 * Gets a copy of the associated Descriptor for the ModelMBeanAttributeInfo
	 *
	 * @return Descriptor associated with the ModelMBeanAttributeInfo object.
	 */
	public Descriptor getDescriptor()
	{
		return descriptor;
	}

	/**
	 * Sets associated Descriptor (full replace) for the
	 * ModelMBeanAttributeDescriptor. If the new Descriptor is null, then the
	 * associated Descriptor reverts to a default descriptor.  The Descriptor
	 * is validated before it is assigned.  If the new Descriptor is invalid,
	 * then an IllegalArgumentException is thrown.
	 *
	 * @param Descriptor inDescriptor - replaces the Descriptor associated
	 * 				with the ModelMBeanAttributeInfo
	 */
	public void setDescriptor(Descriptor inDescriptor)
	{
		this.descriptor = inDescriptor;
	}

	/**
	 * Returns a human readable version of the ModelMBeanAttributeInfo instance
	 */
	public String toString()
	{
		if(descriptor != null)
			return (super.toString() + "\n Descriptor = "+ descriptor.toString());
		else
			return (super.toString() + "\n Descriptor = null");
	}

	//--------------------------- Private methods ---------------------------//

	private void readObject(ObjectInputStream objectinputstream)
					        throws IOException, ClassNotFoundException
	{
		ObjectInputStream.GetField getfield = objectinputstream.readFields();

		try
		{
			descriptor=(DescriptorSupport)getfield.get("attrDescriptor", null);
		}
		catch(Exception exception)
		{
			throw new IOException(exception.toString());
		}
	}

	private void writeObject(ObjectOutputStream objectoutputstream)
														throws IOException
	{
		ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
		putfield.put("attrDescriptor", descriptor);
		putfield.put("currClass", "ModelMBeanAttributeInfo");

		objectoutputstream.writeFields();
	}
}