/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.modelmbean;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamField;
import java.io.IOException;
import java.io.Serializable;
import java.util.Vector;

import javax.management.Descriptor;
import javax.management.MBeanInfo;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanException;
import javax.management.RuntimeOperationsException;

/**
 * This class represents the meta data for ModelMBeans. Descriptors have been
 * added on the meta data objects.
 * <p>
 * Java resources wishing to be manageable instatiate the ModelMBean using the
 * MBeanServer's createMBean method. The resource then sets the ModelMBeanInfo
 * and Descriptors for the ModelMBean instance. The attributes and operations
 * exposed via the ModelMBeanInfo for the ModelMBean are accessible from Mbeans,
 * connectors/adapters like other MBeans. Through the Descriptors, values and
 * methods in the managed application can be defined and mapped to attributes
 * and operations of the ModelMBean. This mapping can be defined during development
 * in a file or dynamically and programmatically at runtime.
 * <p>
 * Every ModelMBean which is instantiated in the MBeanServer becomes manageable:
 * its attributes and operations become remotely accessible through the
 * connectors/adaptors connected to that MBeanServer. A Java object cannot be
 * registered in the MBeanServer unless it is a JMX compliant MBean. By instantiating
 * a ModelMBean, resources are guaranteed that the MBean is valid. MBeanException
 * and RuntimeOperationsException must be thrown on every public method. This
 * allows for wrappering exceptions from distributed communications (RMI, EJB, etc.)
 **/
public class ModelMBeanInfoSupport extends MBeanInfo
								   implements ModelMBeanInfo, Serializable
{
	ModelMBeanAttributeInfo[] attributes = null;
	ModelMBeanConstructorInfo[] constructors = null;
	ModelMBeanOperationInfo[] operations = null;
	ModelMBeanNotificationInfo[] notifications = null;
	Descriptor descriptor = new DescriptorSupport();
	String className = null;
	String description = null;

	private static final long serialVersionUID = 0xc943ccf46e372743L;

	private static final ObjectStreamField[] serialPersistentFields =
	{
		new ObjectStreamField("modelMBeanDescriptor", Descriptor.class),
		new ObjectStreamField("mmbAttributes",new ModelMBeanAttributeInfo[]{}.getClass()),
		new ObjectStreamField("mmbConstructors",MBeanConstructorInfo.class),
		new ObjectStreamField("mmbNotifications",MBeanNotificationInfo.class),
		new ObjectStreamField("mmbOperations",MBeanOperationInfo.class),
		new ObjectStreamField("currClass",java.lang.String.class),
	};

	/**
	 * Constructs a ModelMBeanInfoSupport which is a duplicate of the one passed in.
	 *
	 * @param mbi the ModelMBeanInfo instance from which the ModelMBeanInfo
	 * 				being created is initialized
	 */
	public ModelMBeanInfoSupport(ModelMBeanInfo mbi)
	{
		super();

		ModelMBeanAttributeInfo[] attrs = null;
		ModelMBeanConstructorInfo[] constrs = null;
		ModelMBeanOperationInfo[] opers = null;
		ModelMBeanNotificationInfo[] notifs = null;
		Descriptor descr = null;

		if(mbi.getAttributes() != null)
		{
			ModelMBeanAttributeInfo[] a = (ModelMBeanAttributeInfo[])mbi.getAttributes();
			attrs = new ModelMBeanAttributeInfo[a.length];
			for(int i=0;i<a.length;i++)
				attrs[i] = (ModelMBeanAttributeInfo)a[i].clone();
		}

		if(mbi.getConstructors() != null)
		{
			ModelMBeanConstructorInfo[] c = (ModelMBeanConstructorInfo[])mbi.getConstructors();
			constrs = new ModelMBeanConstructorInfo[c.length];
			for(int i=0;i<c.length;i++)
				constrs[i] = (ModelMBeanConstructorInfo)c[i].clone();

		}

		if(mbi.getOperations() != null)
		{
			ModelMBeanOperationInfo[] o = (ModelMBeanOperationInfo[])mbi.getOperations();
			opers = new ModelMBeanOperationInfo[o.length];
			for(int i=0;i<o.length;i++)
				opers[i] = (ModelMBeanOperationInfo)o[i].clone();
		}

		if(mbi.getNotifications() != null)
		{
			ModelMBeanNotificationInfo[] n = (ModelMBeanNotificationInfo[])mbi.getNotifications();
			notifs = new ModelMBeanNotificationInfo[n.length];
			for(int i=0;i<n.length;i++)
				notifs[i] = (ModelMBeanNotificationInfo)n[i].clone();
		}

		try
		{
			if(mbi.getMBeanDescriptor() != null)
				descr = (Descriptor)mbi.getMBeanDescriptor().clone();
		}
		catch(Exception e)
		{
			if(e instanceof RuntimeException)
				throw (RuntimeException)e;
			throw new RuntimeException();
		}

		this.className = mbi.getClassName();
		this.description = mbi.getDescription();
		this.attributes = attrs;
		this.constructors = constrs;
		this.operations = opers;
		this.notifications = notifs;
		this.descriptor = descr;
	}

	/**
	 * Creates a ModelMBeanInfoSupport with the provided information, but the
	 * descriptor is a default. The default descriptor is: name=mbeanName,
	 * descriptorType=mbean, displayName=this.getClassName(),
	 * persistPolicy=never, log=F,export=F,visiblity=1
	 *
	 * @param className classname of ModelMBeanInfo
	 *
	 * @param description human readable description of the ModelMBean
	 *
	 * @param attributes array of ModelMBeanAttributeInfo objects which have descriptors
	 *
	 * @param constructors array of ModelMBeanConstructorInfo objects which have descriptor
	 *
	 * @param operations array of ModelMBeanOperationInfo objects which have descriptor
	 *
	 * @param notifications array of ModelMBeanNotificationInfo objects which have descriptor
	 */
	public ModelMBeanInfoSupport(String className,
								 String description,
								 ModelMBeanAttributeInfo[] attributes,
								 ModelMBeanConstructorInfo[] constructors,
								 ModelMBeanOperationInfo[] operations,
								 ModelMBeanNotificationInfo[] notifications)
	{
		super(className, description, attributes, constructors, operations, notifications);
		this.attributes = attributes;
		this.constructors = constructors;
		this.operations = operations;
		this.notifications = notifications;
		this.className = className;
		this.description = description;
		appendWithDefaultDescriptors((Descriptor)null);
	}

	/**
	 * Creates a ModelMBeanInfoSupport with the provided information
	 *
	 * @param className classname of ModelMBeanInfo
	 *
	 * @param description human readable description of the ModelMBean
	 *
	 * @param attributes array of ModelMBeanAttributeInfo objects which have descriptors
	 *
	 * @param constructors array of ModelMBeanConstructorInfo objects which have descriptor
	 *
	 * @param operations array of ModelMBeanOperationInfo objects which have descriptor
	 *
	 * @param notifications array of ModelMBeanNotificationInfo objects which have descriptor
	 *
	 * @param mbeandescriptor the descriptor to be used as the MBeanDescriptor
	 * 				containing mbean wide policy. If the descriptor is invalid
	 * 				or null, a default descriptor will be constructed.
	 * 				The default descriptor is:
	 * 				name=mbeanName,
	 * 				descriptorType=mbean,
	 * 				displayName=this.getClassName(),
	 * 				persistPolicy=never,
	 * 				log=F,
	 * 				export=F,
	 * 				visiblity=1.
	 * 				If the descriptor does not contain all these fields,
	 * 				they will be added with these default values.
	 */
	public ModelMBeanInfoSupport(String className,
								 String description,
								 ModelMBeanAttributeInfo[] attributes,
								 ModelMBeanConstructorInfo[] constructors,
								 ModelMBeanOperationInfo[] operations,
								 ModelMBeanNotificationInfo[] notifications,
								 Descriptor mbeandescriptor)
	{
		super(className, description, attributes, constructors, operations, notifications);
		this.attributes = attributes;
		this.constructors = constructors;
		this.operations = operations;
		this.notifications = notifications;
		this.className = className;
		this.description = description;
		this.descriptor = mbeandescriptor;
		appendWithDefaultDescriptors(mbeandescriptor);
	}

	/**
	 * Returns the name of the Java class of the MBean described by this ModelMBeanInfo.
	 *
	 * @return the name of the Java class of the MBean described by this ModelMBeanInfo.
	 */
	public String getClassName()
	{
		return className;
	}

	/**
	 * Returns a human readable description of the MBean. Optional.
	 *
	 * @return a human readable description of the MBean. Optional.
	 */
	public String getDescription()
	{
		return description;
	}

	/**
	 * To clone this ModelMBeanInfo implementation object as
	 * ModelMBeanInfoSupport object.
	 *
	 * @return The duplicate copy of the object
	 */
	public Object clone()
	{
		ModelMBeanAttributeInfo[] attrs = null;
		ModelMBeanConstructorInfo[] constrs = null;
		ModelMBeanOperationInfo[] opers = null;
		ModelMBeanNotificationInfo[] notifs = null;
		Descriptor descr = null;

		if(attributes != null)
		{
			ModelMBeanAttributeInfo[] a = attributes;
			attrs = new ModelMBeanAttributeInfo[a.length];
			for(int i=0;i<a.length;i++)
				attrs[i] = (ModelMBeanAttributeInfo)a[i].clone();
		}

		if(constructors != null)
		{
			ModelMBeanConstructorInfo[] c = constructors;
			constrs = new ModelMBeanConstructorInfo[c.length];
			for(int i=0;i<c.length;i++)
				constrs[i] = (ModelMBeanConstructorInfo)c[i].clone();

		}

		if(operations != null)
		{
			ModelMBeanOperationInfo[] o = operations;
			opers = new ModelMBeanOperationInfo[o.length];
			for(int i=0;i<o.length;i++)
				opers[i] = (ModelMBeanOperationInfo)o[i].clone();
		}

		if(notifications != null)
		{
			ModelMBeanNotificationInfo[] n = notifications;
			notifs = new ModelMBeanNotificationInfo[n.length];
			for(int i=0;i<n.length;i++)
				notifs[i] = (ModelMBeanNotificationInfo)n[i].clone();
		}

		if(descriptor != null)
			descr = (Descriptor)descriptor.clone();

		return new ModelMBeanInfoSupport(getClassName(),
										 getDescription(),
										 attrs,
										 constrs,
										 opers,
										 notifs,
										 descr);
	}

	/**
	 * Gets the attributes info of the ModelMBean targetted object. The return
	 * object will be a array of ModelMBeanAttributeInfo objcts.
	 *
     * @return The array of MBeanAttributeInfo objects are returned after
	 * 				getting the attributes info of ModelMBean targetted object
	 */
	public MBeanAttributeInfo[] getAttributes()
	{
		if(attributes == null)
			return new ModelMBeanAttributeInfo[0];

		return attributes;
	}

	/**
	 * Gets the constructors info of the ModelMBean targetted object.
	 * The return object will be a array of ModelMBeanConstructorInfo objects.
	 *
     * @return The array of MBeanConstructorInfo objects are returned after
	 * 				getting the constructors info of ModelMBean targetted object.
	 */
	public MBeanConstructorInfo[] getConstructors()
	{
		if(constructors == null)
			return new ModelMBeanConstructorInfo[0];

		return constructors;
	}

	/**
	 * Gets the notifications info emitted by the ModelMBean targetted object.
	 * The return object will be a array of ModelMBeanNotificationInfo objects.
	 *
     * @return The array of MBeanNotificationInfo objects are returned after
	 * 				getting the notifications info of the ModelMBean targetted object.
	 */
	public MBeanNotificationInfo[] getNotifications()
	{
		if(notifications == null)
			return new ModelMBeanNotificationInfo[0];

		return notifications;
	}

	/**
	 * Gets the operations info of the ModelMBean targetted object. The return
	 * object will be a array of ModelMBeanOperationInfo objects.
	 *
     * @return The array of MBeanOperationInfo objects are returned after
	 * 				getting the operations info of the ModelMBean targetted object.
	 */
	public MBeanOperationInfo[] getOperations()
	{
		if(operations == null)
			return new ModelMBeanOperationInfo[0];

		return operations;
	}

	/**
	 * Returns a Descriptor array consisting of all Descriptors for the
	 * ModelMBeanInfo including the MBean Descriptor, attribute Descriptors,
	 * operation Descriptors, constructor Descriptors, and notification Descriptors.
	 *
	 * @param inDescriptorType value of descriptorType field that must be set
	 * 				for the descriptor to be returned.  Must be "mbean",
	 * 				"attribute", "operation", "constructor", or "notification".
	 * 				If it is null then all types will be returned.
	 *
	 * @return Descriptor array containing all descriptors for the ModelMBean
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception
	 */
	public Descriptor[] getDescriptors(String inDescriptorType)
							throws MBeanException, RuntimeOperationsException
	{
		Descriptor[] toRet = null;

		if(inDescriptorType == null)
			throw new RuntimeOperationsException(
						new RuntimeException("invalid method param"));

		if(inDescriptorType.equals("mbean"))
		{
			return new Descriptor[]{descriptor};
		}
		else if(inDescriptorType.equals("attribute") && attributes != null)
		{
			 toRet = new Descriptor[attributes.length];
			 for(int i=0;i<attributes.length;i++)
				 toRet[i] = attributes[i].getDescriptor();
		}
		else if(inDescriptorType.equals("operation") && operations != null)
		{
			 toRet = new Descriptor[operations.length];
			 for(int i=0;i<operations.length;i++)
				 toRet[i] = operations[i].getDescriptor();
		}
		else if(inDescriptorType.equals("constructor") && constructors != null)
		{
			 toRet = new Descriptor[constructors.length];
			 for(int i=0;i<constructors.length;i++)
				 toRet[i] = constructors[i].getDescriptor();
		}
		else if(inDescriptorType.equals("notification") && notifications != null)
		{
			 toRet = new Descriptor[notifications.length];
			 for(int i=0;i<notifications.length;i++)
				 toRet[i] = notifications[i].getDescriptor();
		}
		else if(inDescriptorType.equals("all"))
		{
			//toRet = new Descriptor[attributes.length+constructors.length+operations.length+notifications.length];
			Vector descVec = new Vector();
			if(attributes != null)
			{
				for(int i=0; i<attributes.length; i++)
				{
					descVec.add(attributes[i].getDescriptor());
				}
			}
			if(constructors != null)
			{
				for(int i=0; i<constructors.length; i++)
				{
					descVec.add(constructors[i].getDescriptor());
				}
			}
			if(operations != null)
			{
				for(int i=0; i<operations.length; i++)
				{
					descVec.add(operations[i].getDescriptor());
				}
			}
			if(notifications != null)
			{
				for(int i=0; i<notifications.length; i++)
				{
					descVec.add(notifications[i].getDescriptor());
				}
			}

			int size = descVec.size();
			toRet = new Descriptor[size];

			for(int i=0; i<size; i++)
			{
				toRet[i] = (Descriptor)descVec.get(i);
			}
		}
		else
		{
			throw new RuntimeOperationsException(
							new RuntimeException("invalid method param"));
		}

		return toRet;
	}

	/**
	 * Adds or replaces descriptors in the ModelMBeanInfo.
	 *
	 * @param inDescriptors The descriptors to be set in the ModelMBeanInfo.
	 * 				Null elements of the list will be ignored.  All descriptors
	 * 				must have name and descriptorType fields.
	 *
     * @exception MBeanException  Wraps another exception
     *
     * @exception RuntimeOperationsException Wraps exceptions for illegal
	 * 				or null arguments
	 */
	public void setDescriptors(Descriptor[] inDescriptors)
							throws MBeanException, RuntimeOperationsException
	{
		if(inDescriptors == null)
			throw new RuntimeOperationsException(
							new RuntimeException("invalid method param"));

		for(int i=0;i<inDescriptors.length;i++)
			setDescriptor(inDescriptors[i], null);
	}

	/**
	 * Returns a Descriptor requested by name
	 *
	 * @param inDescriptorName The name of the descriptor.
	 *
	 * @return Descriptor containing the descriptor for the ModelMBean with the
	 * 				same name. If a descriptor is not found, null is returned.
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps exceptions for invalid
	 * 				input name or type.
	 */
	public Descriptor getDescriptor(String inDescriptorName)
							throws 	MBeanException,
									RuntimeOperationsException
	{
		Descriptor descr = null;

		if((descr = getDescriptor(inDescriptorName, "mbean")) != null)
			return descr;

		if((descr = getDescriptor(inDescriptorName, "attribute")) != null)
			return descr;

		if((descr = getDescriptor(inDescriptorName, "operation")) != null)
			return descr;

		if((descr = getDescriptor(inDescriptorName, "notification")) != null)
			return descr;

		return null;
	}

	/**
	 * Returns a Descriptor requested by name and descriptorType.
	 *
	 * @param inDescriptorName The name of the descriptor.
	 *
	 * @param inDescriptorType The type of the descriptor being requested.
	 * 				If this is null then all types are searched. Valid types
	 * 				are 'attribute', 'constructor', 'operation', and
	 * 				'notification'. This value must be equal to the
	 * 				'descriptorType' field in the descriptor that is returned.
	 *
	 * @return Descriptor containing the descriptor for the ModelMBean with
	 * 				the same name and descriptorType. If a descriptor is not
	 * 				found, null is returned.
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps exceptions for invalid
	 * 				input name or type.
	 */
	public Descriptor getDescriptor(String inDescriptorName,
									String inDescriptorType)
							throws	MBeanException,
									RuntimeOperationsException
	{
		if(inDescriptorName == null || inDescriptorType == null)
			throw new RuntimeOperationsException(
								new RuntimeException("invalid method param"));

		if(inDescriptorType.equals("mbean"))
		{
			if(descriptor.getFieldValue("name").equals(inDescriptorName))
				return descriptor;
		}
		else if(inDescriptorType.equals("attribute") && attributes != null)
		{
			 for(int i=0;i<attributes.length;i++)
				 if(attributes[i].getDescriptor().getFieldValue("name").equals(inDescriptorName))
					 return attributes[i].getDescriptor();
		}
		else if(inDescriptorType.equals("operation") && operations != null)
		{
			 for(int i=0;i<operations.length;i++)
				 if(operations[i].getDescriptor().getFieldValue("name").equals(inDescriptorName))
					 return operations[i].getDescriptor();
		}
		else if(inDescriptorType.equals("constructor") && constructors != null)
		{
			 for(int i=0;i<constructors.length;i++)
				 if(constructors[i].getDescriptor().getFieldValue("name").equals(inDescriptorName))
					 return constructors[i].getDescriptor();
		}
		else if(inDescriptorType.equals("notification") && notifications != null)
		{
			 for(int i=0;i<notifications.length;i++)
				 if(notifications[i].getDescriptor().getFieldValue("name").equals(inDescriptorName))
					 return notifications[i].getDescriptor();
		}
		else
			throw new RuntimeOperationsException(new RuntimeException("invalid method param"));

		return null;
	}

	/**
	 * Adds or replaces descriptors in all the info arrays for the ModelMBean.
	 *
	 * @param inDescriptor The descriptor to be set in the ModelMBean.
	 * 				It must NOT be null. All descriptors must have name and
	 * 				descriptorType fields.
	 *
	 * @param inDescriptorType The type of the descriptor being set. If this
	 * 				is null then the descriptorType field in the descriptor
	 * 				is used. If specified this value must be set in the
	 * 				descriptorType field for the descriptor to be returned.
	 * 				Must be "mbean","attribute", "operation", "constructor",
	 * 				or "notification".
	 *
	 * @exception RuntimeOperationsException Wraps exceptions for illegal
	 * 				or null arguments.
	 */
	public void setDescriptor(Descriptor inDescriptor, String inDescriptorType)
						throws	MBeanException,
								RuntimeOperationsException
	{
		if(inDescriptor == null || inDescriptor.getFieldValue("name") == null ||
		   inDescriptor.getFieldValue("descriptorType") == null)
			throw new RuntimeOperationsException(new RuntimeException("invalid method param. May be name|descriptorType is absent"));

		String name = (String)inDescriptor.getFieldValue("name");

		if(inDescriptorType == null)
			inDescriptorType = (String)inDescriptor.getFieldValue("descriptorType");

		if(inDescriptorType.equals("mbean"))
		{
			setMBeanDescriptor(inDescriptor);
		}
		else if(inDescriptorType.equals("attribute") && attributes != null)
		{
			 for(int i=0;i<attributes.length;i++)
			 {
				 if(((String)attributes[i].getDescriptor().getFieldValue("name")).equals(name))
					 attributes[i].setDescriptor(inDescriptor);
				 break;
			 }
		}
		else if(inDescriptorType.equals("operation") && operations != null)
		{
			 for(int i=0;i<operations.length;i++)
			 {
				 if(((String)operations[i].getDescriptor().getFieldValue("name")).equals(name))
					 operations[i].setDescriptor(inDescriptor);
				 break;
			 }
		}
		else if(inDescriptorType.equals("constructor") && constructors != null)
		{
			 for(int i=0;i<constructors.length;i++)
			 {
				 if(((String)constructors[i].getDescriptor().getFieldValue("name")).equals(name))
					 constructors[i].setDescriptor(inDescriptor);
				 break;
			 }
		}
		else if(inDescriptorType.equals("notification") && notifications != null)
		{
			 for(int i=0;i<notifications.length;i++)
			 {
				 if(((String)notifications[i].getDescriptor().getFieldValue("name")).equals(name))
					 notifications[i].setDescriptor(inDescriptor);
				 break;
			 }
		}
		else
			throw new RuntimeOperationsException(new RuntimeException("invalid descriptorType"));
	}

	/**
	 * Returns a MBean's descriptor.  This descriptor contains metadata about
	 * the MBean and default policies for persistence and caching for the
	 * entire MBean. Policies may be overridden by descriptors associated with
	 * attribute, constructors, or operations with the same fieldNames.
	 *
     * @return The MBeanDescriptor
     *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception
	 */
	public Descriptor getMBeanDescriptor()
						throws MBeanException, RuntimeOperationsException
	{
		return descriptor;
	}

	/**
	 * Sets the ModelMBean's descriptor (fully replace). This descriptor
	 * contains metadata about the MBean and default policies for persistence
	 * and caching for the entire MBean. Policies may be overridden by
	 * descriptors associated with attribute, constructors, or operations with
	 * the same fieldNames. This operation does a complete replacement of
	 * the descriptor, no merging is done.
	 *
	 * @param  inDescriptor This descriptor contains metadata about the MBean
	 * 				and default policies for persistence and caching for
	 * 				the entire MBean.
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception
	 */
	public void setMBeanDescriptor(Descriptor inDescriptor)
							throws MBeanException, RuntimeOperationsException
	{
		if(inDescriptor == null)
			throw new RuntimeOperationsException(
				new RuntimeException("invalid null Descriptor "));

		inDescriptor.isValid();
		this.descriptor = inDescriptor;

		//set the mbean common properties to attributes/operations/constructors if
		//the field is not present.
		if(attributes != null)
		{
			for(int i=0;i<attributes.length;i++)
			{
				if(attributes[i].getDescriptor().getFieldValue(DescriptorSupport.PERSIST_POLICY) == null && descriptor.getFieldValue(DescriptorSupport.PERSIST_POLICY) != null)
					attributes[i].getDescriptor().setField(DescriptorSupport.PERSIST_POLICY, descriptor.getFieldValue(DescriptorSupport.PERSIST_POLICY));

				if(attributes[i].getDescriptor().getFieldValue(DescriptorSupport.PERSIST_PERIOD) == null && descriptor.getFieldValue(DescriptorSupport.PERSIST_PERIOD) != null)
					attributes[i].getDescriptor().setField(DescriptorSupport.PERSIST_PERIOD, descriptor.getFieldValue(DescriptorSupport.PERSIST_PERIOD));

				if(attributes[i].getDescriptor().getFieldValue(DescriptorSupport.CURRENCY_TIME_LIMIT) == null && descriptor.getFieldValue(DescriptorSupport.CURRENCY_TIME_LIMIT) != null)
					attributes[i].getDescriptor().setField(DescriptorSupport.CURRENCY_TIME_LIMIT, descriptor.getFieldValue(DescriptorSupport.CURRENCY_TIME_LIMIT));

				if(attributes[i].getDescriptor().getFieldValue(DescriptorSupport.VISIBILITY) == null && descriptor.getFieldValue(DescriptorSupport.VISIBILITY) != null)
					attributes[i].getDescriptor().setField(DescriptorSupport.VISIBILITY, descriptor.getFieldValue(DescriptorSupport.VISIBILITY));

				if(attributes[i].getDescriptor().getFieldValue(DescriptorSupport.PRESENTATION_STRING) == null && descriptor.getFieldValue(DescriptorSupport.PRESENTATION_STRING) != null)
					attributes[i].getDescriptor().setField(DescriptorSupport.PRESENTATION_STRING, descriptor.getFieldValue(DescriptorSupport.PRESENTATION_STRING));
			 }
		}

		if(operations != null)
		{
			for(int i=0;i<operations.length;i++)
			{
				if(operations[i].getDescriptor().getFieldValue(DescriptorSupport.CURRENCY_TIME_LIMIT) == null && descriptor.getFieldValue(DescriptorSupport.CURRENCY_TIME_LIMIT) != null)
					operations[i].getDescriptor().setField(DescriptorSupport.CURRENCY_TIME_LIMIT, descriptor.getFieldValue(DescriptorSupport.CURRENCY_TIME_LIMIT));

				if(operations[i].getDescriptor().getFieldValue(DescriptorSupport.VISIBILITY) == null && descriptor.getFieldValue(DescriptorSupport.VISIBILITY) != null)
					operations[i].getDescriptor().setField(DescriptorSupport.VISIBILITY, descriptor.getFieldValue(DescriptorSupport.VISIBILITY));

				if(operations[i].getDescriptor().getFieldValue(DescriptorSupport.PRESENTATION_STRING) == null && descriptor.getFieldValue(DescriptorSupport.PRESENTATION_STRING) != null)
					operations[i].getDescriptor().setField(DescriptorSupport.PRESENTATION_STRING, descriptor.getFieldValue(DescriptorSupport.PRESENTATION_STRING));
			 }
		}

		if(constructors != null)
		{
			for(int i=0;i<constructors.length;i++)
			{
				if(constructors[i].getDescriptor().getFieldValue(DescriptorSupport.VISIBILITY) == null && descriptor.getFieldValue(DescriptorSupport.VISIBILITY) != null)
					constructors[i].getDescriptor().setField(DescriptorSupport.VISIBILITY, descriptor.getFieldValue(DescriptorSupport.VISIBILITY));

				if(constructors[i].getDescriptor().getFieldValue(DescriptorSupport.PRESENTATION_STRING) == null && descriptor.getFieldValue(DescriptorSupport.PRESENTATION_STRING) != null)
					constructors[i].getDescriptor().setField(DescriptorSupport.PRESENTATION_STRING, descriptor.getFieldValue(DescriptorSupport.PRESENTATION_STRING));
			 }
		}

		if(notifications != null)
		{
			for(int i=0;i<notifications.length;i++)
			{
				if(notifications[i].getDescriptor().getFieldValue(DescriptorSupport.LOG) == null && descriptor.getFieldValue(DescriptorSupport.LOG) != null)
					notifications[i].getDescriptor().setField(DescriptorSupport.LOG, descriptor.getFieldValue(DescriptorSupport.LOG));

				if(notifications[i].getDescriptor().getFieldValue(DescriptorSupport.LOG_FILE) == null && descriptor.getFieldValue(DescriptorSupport.LOG_FILE) != null)
					notifications[i].getDescriptor().setField(DescriptorSupport.LOG_FILE, descriptor.getFieldValue(DescriptorSupport.LOG_FILE));

				if(notifications[i].getDescriptor().getFieldValue(DescriptorSupport.VISIBILITY) == null && descriptor.getFieldValue(DescriptorSupport.VISIBILITY) != null)
					notifications[i].getDescriptor().setField(DescriptorSupport.VISIBILITY, descriptor.getFieldValue(DescriptorSupport.VISIBILITY));

				if(notifications[i].getDescriptor().getFieldValue(DescriptorSupport.PRESENTATION_STRING) == null && descriptor.getFieldValue(DescriptorSupport.PRESENTATION_STRING) != null)
					notifications[i].getDescriptor().setField(DescriptorSupport.PRESENTATION_STRING, descriptor.getFieldValue(DescriptorSupport.PRESENTATION_STRING));
			 }
		}
	}

	/**
	 * Returns a ModelMBeanAttributeInfo requested by name.
	 *
	 * @param inName The name of the ModelMBeanAttributeInfo to get.
	 *
     * @return  ModelMBeanAttributeInfo requested by name
     *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception for
	 * 				invalid attribute name or ModelMBeanAttributeInfo to be returned.
	 */
	public ModelMBeanAttributeInfo getAttribute(String inName)
							throws MBeanException, RuntimeOperationsException
	{
		if(inName == null)
			throw new RuntimeOperationsException(new RuntimeException("invalid method param."));

		if(attributes != null)
		{
			for(int i=0;i<attributes.length;i++)
				 if(attributes[i].getName().equals(inName))
					 return attributes[i];
		}

		return null;
	}


	/**
	 * Returns a ModelMBeanOperationInfo requested by name.
	 *
	 * @param  inName The name of the ModelMBeanOperationInfo to get.
	 *
     * @return ModelMBeanOperationInfo requested by name
     *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception for invalid
	 * 				attribute name or ModelMBeanOperationInfo to be returned.
	 */
	public ModelMBeanOperationInfo getOperation(String inName)
							throws MBeanException, RuntimeOperationsException
	{
		if(inName == null)
			throw new RuntimeOperationsException(new RuntimeException("invalid method param."));

		if(operations != null)
		{
			for(int i=0;i<operations.length;i++)
				 if(operations[i].getName().equals(inName))
					 return operations[i];
		}

		return null;
	}

	/**
	 * Returns a ModelMBeanConstructorInfo requested by name.
	 *
	 * @param String inName The name of the ModelMBeanConstructorInfo to get.
	 *
     * @return ModelMBeanConstructorInfo requested by name
     *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception for invalid
	 * 				attribute name or ModelMBeanConstructorInfo to be returned.
	 */
	public ModelMBeanConstructorInfo getConstructor(String inName)
							throws MBeanException, RuntimeOperationsException
	{
		if(inName == null)
			throw new RuntimeOperationsException(new RuntimeException("invalid method param."));

		if(constructors != null)
		{
			for(int i=0;i<constructors.length;i++)
				 if(constructors[i].getName().equals(inName))
					 return constructors[i];
		}

		return null;
	}

	/**
	 * Returns a ModelMBeanNotificationInfo requested by name.
	 *
	 * @param inName The name of the ModelMBeanNotificationInfo to get.
	 *
	 * @return  ModelMBeanNotificationInfo requested by name
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception
	 */
	public ModelMBeanNotificationInfo getNotification(String inName)
							throws MBeanException, RuntimeOperationsException
	{
		if(inName == null)
			throw new RuntimeOperationsException(new RuntimeException("invalid method param."));

		if(notifications != null)
		{
			for(int i=0;i<notifications.length;i++)
				 if(notifications[i].getName().equals(inName))
					 return notifications[i];
		}

		return null;
	}

	//-------------------------- Private methods --------------------------//

	private void appendWithDefaultDescriptors(Descriptor descriptor)
	{
		if(descriptor != null)
			descriptor.isValid();
		else
			descriptor = new DescriptorSupport();

		if(descriptor.getFieldValue("name") == null)
			descriptor.setField("name", "mbeanName");

		if(descriptor.getFieldValue("descriptorType") == null)
			descriptor.setField("descriptorType", "mbean");

		if(descriptor.getFieldValue("displayName") == null)
			descriptor.setField("displayName", getClassName());

		if(descriptor.getFieldValue("persistPolicy") == null)
			descriptor.setField("persistPolicy", "never");

		if(descriptor.getFieldValue("log") == null)
			descriptor.setField("log", "F");

		if(descriptor.getFieldValue("export") == null)
			descriptor.setField("export", "F");

		if(descriptor.getFieldValue("visibility") == null)
			descriptor.setField("visibility", "1");
	}

	private void readObject(ObjectInputStream objectinputstream)
			        			throws IOException, ClassNotFoundException
	{

	    try
	    {
			ObjectInputStream.GetField getfield = objectinputstream.readFields();

			ObjectStreamField[] fields=getfield.getObjectStreamClass().getFields();

			descriptor    = (Descriptor)getfield.get("modelMBeanDescriptor", null);

			attributes    = (ModelMBeanAttributeInfo[])getfield.get("mmbAttributes", null);
			if(attributes==null)
			{
				attributes=(ModelMBeanAttributeInfo[])super.getAttributes();
			}

			constructors  = (ModelMBeanConstructorInfo[])getfield.get("mmbConstructors", null);
			if(constructors==null)
			{
				constructors=(ModelMBeanConstructorInfo[])super.getConstructors();
			}

			notifications = (ModelMBeanNotificationInfo[])getfield.get("mmbNotifications", null);
			if(notifications==null)
			{
				notifications=(ModelMBeanNotificationInfo[])super.getNotifications();
			}

			operations    = (ModelMBeanOperationInfo[])getfield.get("mmbOperations", null);
			if(operations==null)
			{
				operations=(ModelMBeanOperationInfo[])super.getOperations();
			}

			className = (String)getfield.get("currClass","");
	    }
	    catch(Exception exception)
	    {
			exception.printStackTrace();
	        throw new IOException(exception.toString());
	    }
	}

	private void writeObject(ObjectOutputStream objectoutputstream)
														throws IOException
	{
		ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
		putfield.put("modelMBeanDescriptor", descriptor);
		putfield.put("mmbAttributes", attributes);
		putfield.put("mmbConstructors", constructors);
		putfield.put("mmbNotifications", notifications);
		putfield.put("mmbOperations", operations);
		putfield.put("currClass", className);
		objectoutputstream.writeFields();
	}
}