/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.modelmbean;

import javax.management.Attribute;
import javax.management.AttributeChangeNotification;
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.NotificationBroadcaster;
import javax.management.MBeanException;
import javax.management.ListenerNotFoundException;
import javax.management.RuntimeOperationsException;

/**
* This interface must be implemented by the ModelMBeans. An implementation of
* this interface must be shipped with every JMX Agent.
* <p>
* Java resources wishing to be manageable instatiate the ModelMBean using the
* MBeanServer's createMBean method. The resource then sets the ModelMBeanInfo
* (with Descriptors) for the ModelMBean instance. The attributes and operations
* exposed via the ModelMBeanInfo for the ModelMBean are accessible from Mbeans,
* connectors/adapters like other MBeans. Through the ModelMBeanInfo Descriptors,
* values and methods in the managed application can be defined and mapped to
* attributes and operations of the ModelMBean. This mapping can be defined
* during development in an XML formatted file or dynamically and
* programmatically at runtime.
* <p>
* Every ModelMBean which is instantiated in the MBeanServer becomes manageable:
* its attributes and operations become remotely accessible through the
* connectors/adaptors connected to that MBeanServer. A Java object cannot be
* registered in the MBeanServer unless it is a JMX compliant MBean. By
* instantiating a ModelMBean, resources are guaranteed that the MBean is valid.
* <p>
* MBeanException and RuntimeOperationsException must be thrown on every public
* method. This allows for wrappering exceptions from distributed communications
* (RMI, EJB, etc.). These exceptions do not have to be thrown by the
* implementation except in the scenarios described in the specification and javadoc.
*/
public interface ModelMBeanNotificationBroadcaster extends NotificationBroadcaster
{
	/**
	 * Registers an object which implements the NotificationListener interface
	 * as a listener for AttributeChangeNotifications.  This object's
	 * 'handleNotification()' method will be invoked when any
	 * attributeChangeNotification is issued through or by the MBean.  This
	 * does not include other Notifications.  They must be registered for
	 * independently. An AttributeChangeNotification will be generated for this
	 * attributeName.
	 *
	 * @param inlistener The listener object which will handles notifications
	 * 				emitted by the registered MBean.
	 *
	 * @param inAttributeName The name of the MBean attribute for which to
	 * 				receive change notifications. If null, then all attribute
	 * 				changes will cause an attributeChangeNotification to be issued.
	 *
	 * @param inhandback The context to be sent to the listener with the
	 * 				notification when a notification is emitted.
	 *
	 * @exception MBeanException Wrappering exceptions from distributed communications
	 *
	 * @exception RuntimeOPerationsException RuntimeOperationsException
	 * 				to wrapper IllegalArgumentExceptions.
	 *
	 * @exception IllegalArgumentException Listener is null or attributeName is null.
	 */
	public void addAttributeChangeNotificationListener(NotificationListener inlistener,
													   String inAttributeName,
													   Object inhandback)
										throws MBeanException,
											   RuntimeOperationsException,
											   IllegalArgumentException;


	/**
	 * Removes a listener for attributeChangeNotifications from the MBean.
	 *
	 * @param inlistener The listener name which was handling notifications
	 * 				emitted by the registered MBean. This method will remove
	 * 				all information related to this listener.
	 *
	 * @param inAttributeName The attribute for which the listener no longer
	 * 				wants to receive attributeChangeNotifications.
	 *
	 * @exception MBeanException Wrappering exceptions from distributed communications
	 *
	 * @exception RuntimeOperationsException Wrappering exceptions from
	 * 				distributed communications
	 *
	 * @exception ListenerNotFoundException The couple (listener,handback) is
	 * 				not registered in the MBean. The exception message contains
	 * 				either "listener", "handback" or the object name depending
	 * 				on which object cannot be found.
	 */
	public void removeAttributeChangeNotificationListener(NotificationListener inlistener,
														  String inAttributeName)
										throws MBeanException,
											   RuntimeOperationsException,
											   ListenerNotFoundException;


	/**
	 * Sends a Notification which is passed in to the registered Notification
	 * listeners on the ModelMBean as a jmx.modelmbean.general notification.
	 *
	 * @param ntfyObj The notification which is to be passed to the
	 * 				'handleNotification' method of the listener object.
	 *
	 * @exception MBeanException The initializer of the object has thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException:
	 * 				The Notification object passed in parameter is null or invalid.
	 */
	public void sendNotification(Notification ntfyObj)
							throws MBeanException,
									RuntimeOperationsException;

	/**
	 * Sends a Notification which contains the text string that is passed in
	 * to the registered Notification listeners on the ModelMBean.
	 *
	 * @param ntfyText The text which is to be passed in the Notification to
	 * 				the 'handleNotification' method of the listener object.
	 * 				the constructed Notification will be:
	 * 					type        "jmx.modelmbean.general"
	 * 					source      this ModelMBean instance
	 * 					sequence    1
	 *
	 * @exception MBeanException The initializer of the object has thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException:
	 * 				The Notification text string passed in parameter is null or invalid.
	 */
	public void sendNotification(String ntfyText)
							throws MBeanException,
									RuntimeOperationsException;

	/**
	 * Sends an attributeChangeNotification which is passed in to the
	 * registered attributeChangeNotification listeners on the ModelMBean.
	 *
	 * @param ntfyObj The notification which is to be passed to the
	 * 				'handleNotification' method of the listener object.
	 *
	 * @exception MBeanException The initializer of the object has thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException: The
	 * 				Notification object passed in parameter is null or invalid.
	 */
	public void sendAttributeChangeNotification(AttributeChangeNotification ntfyObj)
											throws MBeanException,
												   RuntimeOperationsException;


	/**
	 * Sends an attributeChangeNotification which contains the old value and
	 * new value for the attribute to the registered AttributeChangeNotification
	 * listeners on the ModelMBean.
	 *
	 * @param inOldVal The origional value for the Attribute
	 *
	 * @param inNewVal The current value for the Attribute
	 * <p>
	 * <p>
	 * <PRE>
	 * The constructed attributeChangeNotification will be:
	 * type        "jmx.attribute.change"
	 * source      this ModelMBean instance
	 * sequence    1
	 * attributeName oldValue.getName()
	 * attributeType oldValue's class
	 * attributeOldValue oldValue.getValue()
	 * attributeNewValue newValue.getValue()
	 * </PRE>
	 * <p>
	 *
	 * @exception MBeanException to wrapper implementation exceptions
	 *
	 * @exception RuntimeOperationsException to wrapper IllegalArgumentExceptions.
	 */
	public void sendAttributeChangeNotification(Attribute inOldVal, Attribute inNewVal)
											throws MBeanException,
												   RuntimeOperationsException;
}