/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.modelmbean;

import java.lang.reflect.Method;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamField;
import java.io.IOException;

import javax.management.Descriptor;
import javax.management.DescriptorAccess;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;

/**
 * The ModelMBeanOperationInfo object describes a management operation
 * of the ModelMBean. It is a subclass of MBeanOperationInfo with the
 * addition of an associated Descriptor and an implementation of the
 * DescriptorAccess interface.
 * <P>
 * <P>
 * <PRE>
 * The fields in the descriptor are defined, but not limited to, the following:
 *	name           : operation name
 *	descriptorType : must be "operation"
 *	class          : class where method is defined (fully qualified)
 *	role           : getter, setter, operation, constructor
 *	targetObject   : object on which to execute this method
 *	targetType     : type of object reference for targetObject:
 *
 *                ObjectReference|EJBHandle|IOR|RMIReference
 *	lastReturnedValue     : cached value for operation
 *	currencyTimeLimit     : how long cached value is valid
 *	lastReturnedTimeStamp : when cached value was set
 *	visibility            : 1-4 where 1: always visible 4: rarely visible
 *	presentationString    :  xml formatted string to describe how to present operation
 * </PRE>
 * <P> The default descriptor will have name, descriptorType, and role fields set.
 */

public class ModelMBeanOperationInfo extends MBeanOperationInfo
									implements DescriptorAccess
{
	Descriptor descriptor = new DescriptorSupport();

	private static final long serialVersionUID = 0x7e1dce2a56f1e767L;

	private static final ObjectStreamField[] serialPersistentFields =
	{
		new ObjectStreamField("operationDescriptor", DescriptorSupport.class),
		new ObjectStreamField("currClass",java.lang.String.class)
	};


	/**
	 * Constructs a new ModelMBeanOperationInfo object with default descriptor
	 *
	 * @param method  The java.lang.reflect.Method object describing the MBean operation.
	 *
	 * @param description  A human readable description of the operation.
	 */
	public ModelMBeanOperationInfo (String description, Method method)
	{
		super(description, method);
	}

	/**
	 * Constructs a new ModelMBeanOperationInfo object
	 * @param method  The java.lang.reflect.Method object describing the MBean operation.
	 * @param description  A human readable description of the operation.
	 * @param descriptor Descriptor containing the appropriate metadata for this
	 * instance of the MBeanNotificationInfo.If it is null or invalid then a default
	 * desriptor will be created.
	 */
	public ModelMBeanOperationInfo (String description,
									Method method,
									Descriptor descriptor)
	{
		super(description, method);
		this.descriptor = descriptor;
	}

	/**
	* Constructs a ModelMBeanOperationInfo object with a default descriptor.
	*
	* @param name The method name
	*
	* @param description  A human readable description of the operation.
	*
	* @param signature The array of MBeanParameterInfo contains the parameters
	* 				or arguments of the method
	*
	* @param type The return type of the method
	*
	* @param impact The impact of the method, will be anyone of the following
	* 				INFO, ACTION, ACTION_INFO,  UNKNOWN.
	*/
	public ModelMBeanOperationInfo (String name,
					 				String description,
					 				MBeanParameterInfo[] signature,
					 				String type,
					 				int impact)
	{
		super(name, description, signature, type, impact);
	}

	/**
	 * Constructs a ModelMBeanOperationInfo object escriptorth with
	 * the specified Descriptor.
	 *
	 * @param name The method name
	 *
	 * @param description  A human readable description of the operation.
	 *
	 * @param signature The array of MBeanParameterInfo contains the parameters
	 * 				or arguments of the method
	 *
	 * @param type The return type of the method
	 *
	 * @param impact The impact of the method, will be anyone of the following
	 * 				INFO, ACTION, ACTION_INFO,  UNKNOWN.
	 *
	 * @param descriptor Descriptor containing the appropriate metadata for
	 * 				this instance of the MBeanOperationInfo. If it is null or
	 * 				invalid then a default desriptor will be created.
	 */
	public ModelMBeanOperationInfo (String name,
								 	String description,
								 	MBeanParameterInfo[] signature,
								 	String type,
								 	int impact,
								 	Descriptor descriptor)
	{
		super(name, description, signature, type, impact);
		this.descriptor = descriptor;
	}

	/**
	 * Creates a ModelMBeanOperationInfo with the specified
	 * ModelMBeanOperationInfo object.
	 *
	 * @param inInfo ModelMBeanOperationInfo object
	 */
	public ModelMBeanOperationInfo(ModelMBeanOperationInfo inInfo)
	{
		super(inInfo.getName(), inInfo.getDescription(), inInfo.getSignature(),
			  inInfo.getReturnType(), inInfo.getImpact());
		if(inInfo.getDescriptor() != null)
			this.descriptor = (Descriptor)inInfo.getDescriptor().clone();
	}

	/**
	 * Creates a duplicate object of ModelMBeanOperationInfo
	 *
	 * @return Create and return a duplicate object of ModelMBeanOperationInfo
	 */
	public Object clone()
	{
		Descriptor clonedDescr = null;

		if(descriptor != null)
			clonedDescr = (Descriptor)descriptor.clone();

		return new ModelMBeanOperationInfo(getName(), getDescription(),
										   getSignature(), getReturnType(),
										   getImpact(), clonedDescr);
	}

	/**
	 * Gets the corresponding Descriptor of this ModelMBeanNotificationInfo
	 *
	 * @return This returns the Descriptor associated with
	 * 				this ModelMBeanNotificationInfo
	 */
	public Descriptor getDescriptor()
	{
		return descriptor;
	}

    /**
     * Sets the specified Descriptor to this ModelMBeanNotificationInfo.If the
     * value is null then default Descriptor will be taken.  The Descriptor is
	 * validated before it is assigned. If the new Descriptor is invalid, then
	 * an IllegalArgumentException is thrown.
	 *
     * @param inDescriptor This replaces the Descriptor associated with the
     * 				ModelMBeanNotification interface
     *
     * @throws IllegalArgumentException If the new Descriptor is invalid,
	 * 				then it will be thrown.
     */
	public void setDescriptor(Descriptor inDescriptor)
	{
		this.descriptor = inDescriptor;
	}

	/**
	 * Returns a human readable version of the ModelMBeanOperationInfo instance
	 *
	 * @return This returns human readable version of the
	 * 				ModelMBeanOperationInfo instance
	 */
	public String toString()
	{
		if(descriptor != null)
			return (super.toString() + "\n Descriptor = "+ descriptor.toString());
		else
			return (super.toString() + "\n Descriptor = null");
	}

	//--------------------------- Private methods ---------------------------//

	private void readObject(ObjectInputStream objectinputstream)
	    						throws IOException, ClassNotFoundException
	{
	    ObjectInputStream.GetField getfield = objectinputstream.readFields();

	    try
	    {
	        descriptor=(DescriptorSupport)getfield.get("operationDescriptor", null);
	    }
	    catch(Exception exception)
	    {
	        //throw new IOException(malformedobjectnameexception.toString());
	    }
	}

	private void writeObject(ObjectOutputStream objectoutputstream)
					          							throws IOException
	{
		ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
		putfield.put("operationDescriptor", descriptor);
		putfield.put("currClass", "ModelMBeanOperationInfo");

		objectoutputstream.writeFields();
	}
}