/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.monitor;

import java.util.Date;

import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanException;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.ReflectionException;

import com.adventnet.agent.logging.Log;
import com.adventnet.agent.logging.LogFactory;

/**
 * Defines a monitor MBean designed to observe the values of a string attribute.
 * <P>
 * A string monitor sends notifications as follows:
 * <UL>
 * <LI> if the attribute value matches the string to compare value,
 *      a {@link MonitorNotification#STRING_TO_COMPARE_VALUE_MATCHED match notification} is sent.
 *      The notify match flag must be set to <CODE>true</CODE>.
 *      <BR>Subsequent matchings of the string to compare values do not cause
 * 		further notifications unless the attribute value differs from the
 * 		string to compare value.
 * <LI> if the attribute value differs from the string to compare value,
 *      a {@link MonitorNotification#STRING_TO_COMPARE_VALUE_DIFFERED differ notification} is sent.
 *      The notify differ flag must be set to <CODE>true</CODE>.
 *      <BR>Subsequent differences from the string to compare value do not
 * 		cause further notifications unless the attribute value matches the
 * 		string to compare value.
 * </UL>
 */
public class StringMonitor extends Monitor implements StringMonitorMBean
{
    /**
     * String to compare with the observed attribute.
     * <BR>The default value is an empty character sequence.
     */
    private String string_to_compare = new String();

    /**
     * Flag indicating if the string monitor notifies when matching the string
	 * to compare. <BR>The default value is set to <CODE>false</CODE>.
     */
	private	boolean	notify_match = false;

    /**
     * Flag indicating if the string monitor notifies when differing from the
	 * string to compare. <BR>The default value is set to <CODE>false</CODE>.
     */
	private	boolean	notify_differ =	false;

    /**
     * Derived gauge.
     */
    private transient String derivedGauge = new String();

    /**
     * Derived gauge timestamp.
     */
    private transient long derivedGaugeTimestamp = (new Date()).getTime();

	private	Thread stringMonitorThread		=	null;
	private	long sequenceNumber				=	1;
	private	boolean nmatch					=	true;
	private	boolean	ndiffer					=	true;
	private	MonitorNotification	notif		=	null;
	private	int	ERROR_VALUE					=	0;
	private ObjectName objName				= 	null;
	private Log log;

	/**
	 * Default Constructor.
	 */
	public StringMonitor()
	{
		createLogger();
	}

	/**
	 * This method gets the value of the derived gauge. The derived gauge is
	 * either the exact value of the observed attribute , or the difference
	 * between the two consecutive observed values of the attribute.
	 *
	 * @return An instance of java.lang.String giving the value of the derived gauge.
	 */
	public String getDerivedGauge()
	{
		return derivedGauge;
	}

	/**
	 * This method gets the value of the derived gauge time stamp.The derived
	 * gauge time stamp is the value(in the nearest miliseconds) when the
	 * notification was triggered.
	 *
	 * @return long value representing the time the notification was triggered.
	 */
	public long getDerivedGaugeTimeStamp()
	{
		return derivedGaugeTimeStamp;
	}

	/**
	 * This method sets the differing notification's on/off switch value.
	 *
	 * @return  The differing notification's on/off switch value.
	 */
	public boolean getNotifyDiffer()
	{
		return notify_differ;
	}

	/**
	 * This method sets the differing notification's on/off switch value.
	 *
	 * @param value - The differing notification's on/off switch value.
	 */
	public void setNotifyDiffer(boolean value)
	{
		this.notify_differ = value;
	}

	/**
	 * This method gets the matching notification's on/off switch value.
	 *
	 * @return The matching notification's on/off switch value.
	 */
	public boolean getNotifyMatch()
	{
		return notify_match;
	}

	/**
	 * This method sets the matching notification's on/off switch value.
	 *
	 * @param value - The matching notification's on/off switch value.
	 */
	public void setNotifyMatch(boolean value)
	{
		this.notify_match = value;
	}

	/**
	 * This method gets the string to compare with the observed attribute.
	 *
	 * @return the String to be compared.
	 */
	public String getStringToCompare()
	{
		return string_to_compare;
	}

	/**
	 * This method sets the string to compare with the observed attribute.
	 *
	 * @param The String Value to be compared.
	 *
	 * @exception java.lang.IllegalArgumentException - The specified string to
	 * 				compare is null.
	 */
	public void setStringToCompare(String value) throws IllegalArgumentException
	{
		if(value == null)
			throw new IllegalArgumentException("The String to Compare cannot be null");

		this.string_to_compare = value;
	}

	/**
	 * This method starts the String Monitor.
	 */
	public void start()
	{
		stringMonitorThread = new Thread(new StringMonitorThread(this));
		isActive = true;
		stringMonitorThread.setName("StringMonitorThread-"+stringMonitorThread.getName());
		stringMonitorThread.start();
		log.trace("String Monitor Started");
	}

	/**
	 * This method stops the String Monitor.
	 */
	public void stop()
	{
		isActive = false;
		stringMonitorThread.stop();
		log.trace("String Monitor Stopped");
	}

	/**
	 * Overriding the preRegister() of MBeanRegistration interface to store
	 * the ObjectName locally.
	 */
	public ObjectName preRegister(MBeanServer server, ObjectName name)
													throws Exception
	{
		this.objName = name;
		super.preRegister(server,name);
		return name;
	}

	public MBeanNotificationInfo[] getNotificationInfo()
	{
		MBeanNotificationInfo[] notifInfo = super.getNotificationInfo();

		MBeanNotificationInfo[] newNotifInfo = new MBeanNotificationInfo[notifInfo.length+1];

		for(int i=0;i<notifInfo.length;i++)
		{
			newNotifInfo[i] = notifInfo[i];
		}

		String[] types = {"jmx.monitor.string.matches","jmx.monitor.string.differs"};
		newNotifInfo[newNotifInfo.length - 1] = new MBeanNotificationInfo(types,"Monitor String Comparison Notification","Emimitted for a difference in the comparison ");


		return newNotifInfo;
	}

	//---------------------------- Private methods ---------------------------//

	private void createLogger()
	{
		try
		{
			log = LogFactory.getInstance("JMX");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	//------------------------------ Inner class -----------------------------//

	// This inner class implements Runnable and when the StringMonitor is started,
	// the start method of this class is called and it periodically polls the
	// Observed Attribute according to the granularity period and when there is
	// a deviation from the configuration details, sends a notification.
	class StringMonitorThread implements Runnable
	{
		StringMonitor monitor = null;
		private boolean isNotifDifferSent = false;
		private boolean isNotifMatchSent  = false;

		StringMonitorThread(StringMonitor monitor)
		{
			this.monitor = monitor;
		}

		public void run()
		{
			Object obj = null;
			boolean temp = true;

			while(true)
			{
				try
				{
					obj = server.getAttribute(observedObject,attributeName);
					derivedGaugeTimeStamp = System.currentTimeMillis();
				}
				catch(InstanceNotFoundException ine)
				{
					if(Monitor.OBSERVED_OBJECT_ERROR_NOTIFIED == 1)
					{
						sendNotification(new MonitorNotification(
								MonitorNotification.OBSERVED_OBJECT_ERROR,
								monitor, observedObject, attributeName,
								null, null, sequenceNumber++));
						Monitor.OBSERVED_OBJECT_ERROR_NOTIFIED = 0;
					}
					sleep();
					continue;
				}
				catch(AttributeNotFoundException ane)
				{
					 if(Monitor.OBSERVED_ATTRIBUTE_ERROR_NOTIFIED == 1)
					 {
						 sendNotification(new MonitorNotification(
						 		MonitorNotification.OBSERVED_ATTRIBUTE_ERROR,
								 monitor, observedObject, attributeName,
								 null, null, sequenceNumber++));
						 Monitor.OBSERVED_ATTRIBUTE_ERROR_NOTIFIED = 0;
					 }
					 sleep();
					 continue;
				}
				catch(MBeanException mbe)
				{
					if(Monitor.RUNTIME_ERROR_NOTIFIED == 1)
					{
						 sendNotification(new MonitorNotification(
						 		MonitorNotification.RUNTIME_ERROR,
								 monitor, observedObject, attributeName,
								 null, null, sequenceNumber++));
						 Monitor.RUNTIME_ERROR_NOTIFIED = 0;
					}

					/*
					if(Monitor.OBSERVED_ATTRIBUTE_TYPE_ERROR_NOTIFIED  == 1)
					{
						sendNotification(new MonitorNotification(MonitorNotification.OBSERVED_ATTRIBUTE_TYPE_ERROR,monitor,
						observedObject,attributeName,															 null,null,sequenceNumber++));
						Monitor.OBSERVED_ATTRIBUTE_TYPE_ERROR_NOTIFIED = 0;
					}*/
					sleep();
					continue;
				}
				catch(ReflectionException re)
				{
					 if(Monitor.RUNTIME_ERROR_NOTIFIED == 1)
					 {
						 sendNotification(new MonitorNotification(
						 			MonitorNotification.RUNTIME_ERROR,
									monitor, observedObject, attributeName,
									null, null, sequenceNumber++));
						 Monitor.RUNTIME_ERROR_NOTIFIED = 0;
					 }
					 sleep();
					 continue;
				}
				catch(Exception e)
				{
					 if(Monitor.RUNTIME_ERROR_NOTIFIED == 1)
					 {
						 sendNotification(new MonitorNotification(
						 			MonitorNotification.RUNTIME_ERROR,
									 monitor, observedObject, attributeName,
									 null, null, sequenceNumber++));
						 Monitor.RUNTIME_ERROR_NOTIFIED = 0;
					 }
					 sleep();
					 continue;
				}

				if(obj == null)
				{
					continue;
				}

				if(!(obj.getClass().getName().equals("java.lang.String")))
				{
					 if(Monitor.OBSERVED_ATTRIBUTE_TYPE_ERROR_NOTIFIED == 1)
					 {
						 sendNotification(new MonitorNotification(
						 		MonitorNotification.OBSERVED_ATTRIBUTE_TYPE_ERROR,
								 monitor, observedObject, attributeName,
								 null, null, sequenceNumber++));
						 Monitor.OBSERVED_ATTRIBUTE_TYPE_ERROR_NOTIFIED = 0;
					 }
				}

				derivedGauge = (String)obj;

				if(notify_match)
				{
					if(isNotifMatchSent)
					{
						if(!(derivedGauge.equals(string_to_compare)))
						{
							isNotifMatchSent = false;
						}
					}
					else
					{
						if(derivedGauge.equals(string_to_compare))
						{
							notif = new MonitorNotification(
									MonitorNotification.STRING_TO_COMPARE_VALUE_MATCHED,
									monitor, observedObject, attributeName,
									derivedGauge, string_to_compare, sequenceNumber++);

							isNotifMatchSent = true;
							sendNotification(notif);
						}
					}
				}

				if(notify_differ)
				{
					if(isNotifDifferSent)
					{
						if(derivedGauge.equals(string_to_compare))
						{
							isNotifDifferSent = false;
						}
					}
					else
					{
						if(!(derivedGauge.equals(string_to_compare)))
						{
							notif = new MonitorNotification(
								MonitorNotification.STRING_TO_COMPARE_VALUE_DIFFERED,
								monitor, observedObject, attributeName,
								derivedGauge, string_to_compare, sequenceNumber++);

							isNotifDifferSent = true;
							sendNotification(notif);
						}
					}
				}

				try
				{
					Thread.sleep(granularityPeriod);
				}
				catch(InterruptedException IE)
				{
				}
			}
		}

		private void sleep()
		{
			try
			{
				Thread.sleep(granularityPeriod);
			}
			catch(Exception e)
			{
			}
		}
	}//End StringMonitorThread class.
}//End StringMonitor class.