/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.util.Vector;

import javax.management.ObjectName;
import javax.management.Notification;
import javax.management.MBeanServerNotification;
import javax.management.NotificationFilterSupport;

/**
 * This filter allows to filter MBeanServerNotification notifications by
 * selecting the ObjectNames of interest and the operations (registration,
 * unregistration, both) of interest (corresponding to notification types).
 */
public class MBeanServerNotificationFilter extends NotificationFilterSupport
{
	// This variable manages a list of ObjectNames .
	private Vector mEnabledObjectNames = null;
	private Vector mDisabledObjectNames = null;

	private boolean mEnableAll = true;
	private boolean mDisableAll = false;

	/**
	 * Creates a filter selecting all MBeanServerNotification notifications for
	 * all ObjectNames
	 */
	public MBeanServerNotificationFilter()
	{
		mEnabledObjectNames = new Vector();
		mDisabledObjectNames = new Vector();
	}

	/**
	 * Disables any MBeanServerNotification (i.e. all ObjectNames deselected)
	 */
	public void disableAllObjectNames()
	{
		mEnableAll = false;
		mDisableAll = true;
	}

	/**
	 * Disables MBeanServerNotifications concerning given ObjectName.
	 *
	 * @param theObjName - ObjectName no longer of interest
	 *
	 * @exception java.lang.IllegalArgumentException - if the given ObjectName is null
	 */
	public void disableObjectName(ObjectName theObjName)
										throws IllegalArgumentException
	{
		mEnableAll = false;
		mDisabledObjectNames.add(theObjName);
	}

	/**
	 * Enables all MBeanServerNotifications (i.e. all ObjectNames selected)
	 */
	public void enableAllObjectNames()
	{
		mEnableAll = true;
	}

	/**
	 *  Enables MBeanServerNotifications concerning given ObjectName.
	 *
	 *  @param theObjName - ObjectName of interest
	 *
	 *  @exception java.lang.IllegalArgumentException - if the given ObjectName is null
	 */
	public void enableObjectName(ObjectName theObjName)
											throws IllegalArgumentException
	{
		if(theObjName == null)
			throw new IllegalArgumentException("ObjectName Cannot be null");

		mEnableAll = false;

		//TODO -- Check if it is already present. If yes, then do not add.
		mEnabledObjectNames.add(theObjName);
	}

	/**
	 * Gets all the ObjectNames disabled.
	 *
	 * @return Vector of ObjectNames:
	 * 				-null means all ObjectNames are implicitly deselected,
	 * 					except the ObjectNames explicitly selected
	 * 				-empty means all ObjectNames are selected, i.e. no
	 * 					ObjectName deselected.
	 */
	public Vector getDisabledObjectNames()
	{
		if(mEnableAll)
			return new Vector();

		return mDisabledObjectNames;
	}

	/**
	 * Gets all the ObjectNames enabled.
	 *
	 * @return Vector of ObjectNames:
	 * 				-null means all ObjectNames are implicitly selected,
	 * 					except the ObjectNames explicitly deselected
     * 				- empty means all ObjectNames are deselected, i.e. no
	    * 				ObjectName selected.
	 */
	public Vector getEnabledObjectNames()
	{
		if(mEnableAll)
			return null;

		return mEnabledObjectNames;
	}

	/**
	 * Invoked before sending the specified notification to the listener.
	 * If:
	 *  	- the ObjectName of the concerned MBean is selected (explicitly OR
	 * 			(implicitly and not explicitly deselected))
	 * 		AND
	 * 		- the type of the operation (registration or unregistration) is
	 * 			selected then the notification is sent to the listener.
	 *
	 * Overrides:
	 *   isNotificationEnabled in class NotificationFilterSupport
	 *
	 * @param theNtf - The notification to be sent.
	 *
	 * @return true if the notification has to be sent to the listener, false otherwise.
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 */
	public boolean isNotificationEnabled(Notification theNtf)
											throws IllegalArgumentException
	{
		if(theNtf == null)
			throw new IllegalArgumentException();

		String type = theNtf.getType();
		ObjectName name = null;

		if(theNtf instanceof MBeanServerNotification)
		{
			//Now you have to check if it already present in the enabled type of this filter.
			//Check if the ObjectName is present in the list of deselected .
			//If it is not , send the Notification.
			name = ((MBeanServerNotification)(theNtf)).getMBeanName();
			if(mEnableAll)
			{
				return true;
			}
			else
			{
				//if it is present in the list of enabled ObjectNames, return true.else return false.
				if(mEnabledObjectNames.contains(name))
				{
					return true;
				}
			}
		}

		return false;
	}
}