/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.util.Map;
import java.util.List;

import javax.management.ObjectName;

/**
 * This interface has to be implemented by any MBean class expected to represent
 * a relation managed using the Relation Service.
 * Simple relations, i.e. having only roles, no properties or methods, can be
 * created directly by the Relation Service (represented as RelationSupport
 * objects, internally handled by the Relation Service).
 * If the user wants to represent more complex relations, involving properties
 * and/or methods, he has to provide his own class implementing the Relation
 * interface. This can be achieved either by inheriting from RelationSupport
 * class, or by implementing the interface (fully or delegation to a
 * RelationSupport object member).
 * Specifying such user relation class is to introduce properties and/or methods.
 * Those have to be exposed for remote management. So this means that any user
 * relation class must be a MBean class.
 */
public interface Relation
{
	/**
	 * Returns all roles present in the relation
	 *
	 * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully retrieved) and a RoleUnresolvedList
	 * 				(for roles not readable).
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 */
	public RoleResult getAllRoles()
                       throws RelationServiceNotRegisteredException;


	/**
	 * Retrieves MBeans referenced in the various roles of the relation.
	 *
	 * @return a HashMap mapping:
	 * 				ObjectName -> ArrayList of String (role names)
	 */
	public Map getReferencedMBeans();


	/**
	 * Returns relation identifier (used to uniquely identify the relation
	 * inside the Relation Service)
	 *
	 * @return This will return the identifier
	 */
	public String getRelationId();


	/**
	 * Returns ObjectName of the Relation Service handling the relation
	 *
	 * @return This returns the object name of the relation service
	 */
	public ObjectName getRelationServiceName();

	/**
	 * Returns name of associated relation type.
	 *
	 * @return Name of associated relation type.
	 */
	public String getRelationTypeName();

	/**
	 * Retrieves role value for given role name.
	 * Checks if the role exists and is readable according to the relation type.
	 *
	 * @param theRoleName - name of role
	 *
	 * @return the ArrayList of ObjectName objects being the role value
	 *
	 * @throws java.lang.IllegalArgumentException - if null role name
	 *
	 * @throws RoleNotFoundException - if:
	 * 				- there is no role with given name
	 * 				- the role is not readable.
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 */
	public List getRole(String theRoleName)
                       throws IllegalArgumentException,
                              RoleNotFoundException,
                              RelationServiceNotRegisteredException;

	/**
	 * Returns the number of MBeans currently referenced in the given role.
	 *
	 * @param theRoleName - name of role
	 *
	 * @return the number of currently referenced MBeans in that role
	 *
	 * @throws java.lang.IllegalArgumentException - if null role name
	 *
	 * @throws RoleNotFoundException - if there is no role with given name
	 */
	public Integer getRoleCardinality(String theRoleName)
                                     throws IllegalArgumentException,
                                            RoleNotFoundException;


	/**
	 * Retrieves values of roles with given names.
	 * Checks for each role if it exists and is readable according to the
	 * relation type.
	 *
	 * @param theRoleNameArray - array of names of roles to be retrieved
	 *
	 * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully retrieved) and a RoleUnresolvedList
	 * 				(for roles not retrieved).
	 *
	 * @throws java.lang.IllegalArgumentException - if null role name
	 *
	 * @throws  RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 */
	public RoleResult getRoles(String[] theRoleNameArray)
                    throws IllegalArgumentException,
                           RelationServiceNotRegisteredException;


	/**
	 * Callback used by the Relation Service when a MBean referenced in a role
	 * is unregistered. The Relation Service will call this method to let the
	 * relation take action to reflect the impact of such unregistration.
	 * BEWARE. the user is not expected to call this method.
	 * Current implementation is to set the role with its current value
	 * (list of ObjectNames of referenced MBeans) without the unregistered one.
	 *
	 * @param theObjName - ObjectName of unregistered MBean
	 *
	 * @param theRoleName - name of role where the MBean is referenced
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RoleNotFoundException - if role does not exist in the relation
	 * 				or is not writable
	 *
	 * @throws InvalidRoleValueException - if role value does not conform to
	 * 				the associated role info (this will never happen when
	 * 				called from the Relation Service)
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws RelationTypeNotFoundException - if the relation type has not
	 * 				been declared in the Relation Service.
	 *
	 * @throws RelationNotFoundException - if this method is called for a
	 * 				relation MBean not added in the Relation Service.
	 */
	public void handleMBeanUnregistration(ObjectName theObjName,
                                      String theRoleName)
                               throws IllegalArgumentException,
                                      RoleNotFoundException,
                                      InvalidRoleValueException,
                                      RelationServiceNotRegisteredException,
                                      RelationTypeNotFoundException,
                                      RelationNotFoundException;

	/**
	 * Returns all roles in the relation without checking read mode
	 * @return a RoleList
	 */
	public RoleList retrieveAllRoles();

	/**
	 * Sets the given role.
	 * Will check the role according to its corresponding role definition
	 * provided in relation's relation type will send a notification
	 * (RelationNotification with type RELATION_BASIC_UPDATE or
	 * RELATION_MBEAN_UPDATE, depending if the relation is a MBean or not).
	 *
	 * @param theRole - role to be set (name and new value)
	 *
	 * @throws java.lang.IllegalArgumentException - if null role
	 *
	 * @throws RoleNotFoundException - if the role is not writable (no test on
	 * 				the write access mode performed when initialising the role)
	 *
	 * @throws InvalidRoleValueException - if value provided for role is not
	 * 				valid, i.e.:
	 * 				- the number of referenced MBeans in given value is less
	 * 					than expected minimum degree
	 * 				- the number of referenced MBeans in provided value exceeds
	 * 					expected maximum  degree
	 * 				- one referenced MBean in the value is not an Object of the
	 * 					MBean class expected for that role
	 * 				- a MBean provided for that role does not exist
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws RelationTypeNotFoundException - if the relation type has not
	 * 				been declared in the Relation Service.
	 */
	public void setRole(Role theRole)
             throws IllegalArgumentException,
                    RoleNotFoundException,
                    InvalidRoleValueException,
                    RelationServiceNotRegisteredException,
                    RelationTypeNotFoundException,
                    RelationNotFoundException;

	/**
	 * Sets the given roles.
	 * Will check the role according to its corresponding role definition
	 * provided in relation's relation type will send one notification
	 * (RelationNotification with type RELATION_BASIC_UPDATE or
	 * RELATION_MBEAN_UPDATE, depending if the relation is a MBean or not) per
	 * updated role.
	 *
	 * @param theRoleList - list of roles to be set
	 *
	 * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully set) and a RoleUnresolvedList (for roles not set).
	 *
	 * @throws java.lang.IllegalArgumentException - if null role name
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws RelationTypeNotFoundException - if the relation type has not
	 * 				been declared in the Relation Service.
	 *
	 * @throws RelationNotFoundException - if the relation MBean has not been
	 * 				added in the Relation Service.
	 */
	public RoleResult setRoles(RoleList theRoleList)
                    throws IllegalArgumentException,
                           RelationServiceNotRegisteredException,
                           RelationTypeNotFoundException,
                           RelationNotFoundException;
}