/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.util.List;
import java.util.Map;

import javax.management.ObjectName;
import javax.management.InstanceNotFoundException;

/**
 * The Relation Service is in charge of creating and deleting relation types
 * and relations, of handling the consistency and of providing query mechanisms.
 */
public interface RelationServiceMBean
{
	/**
	 * Adds a MBean created by te user (and registered by him in the MBean
	 * Server) as a relation in the Relation Service.
	 *      To be added as a relation, the MBean must conform to the following:
	 *      - implement the Relation interface
	 *      - have for RelationService ObjectName the ObjectName of current Relation Service
	 *      - have a relation id unique and unused in current Relation Service
	 *      - have for relation type a relation type created in the Relation Service
	 *      - have roles conforming to the role info provided in the relation type.
	 *
	 * @param theRelObjectName - ObjectName of the relation MBean to be added.
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws java.lang.NoSuchMethodException - If the MBean does not implement
	 * 				the Relation interface
	 *
	 * @throws InvalidRelationIdException - if:
	 *           - no relation identifier in MBean
	 *           - the relation identifier is already used in the Relation Service
	 *
	 * @throws InstanceNotFoundException - if the MBean for given ObjectName
	 * 				has not been registered
	 *
	 * @throws InvalidRelationServiceException - if:
	 *           - no Relation Service name in MBean
	 *           - the Relation Service name in the MBean is not the one of
	 * 					the current Relation Service
	 *
	 * @throws RelationTypeNotFoundException - if:
	 *           - no relation type name in MBean
	 *           - the relation type name in MBean does not correspond to a
	 * 					relation type created in the Relation Service
	 *
	 * @throws InvalidRoleValueException - if:
	 *           - the number of referenced MBeans in a role is less than
	 * 					expected minimum degree
	 *           - the number of referenced MBeans in a role exceeds expected
	 * 					maximum degree
	 *           - one referenced MBean in the value is not an Object of the
	 * 					MBean class expected for that role
	 *           - a MBean provided for a role does not exist
	 *
	 * @throws RoleNotFoundException - if a value is provided for a role that
	 * 				does not exist in the relation type
	 */
	public void addRelation(ObjectName theRelObjectName)
                 throws IllegalArgumentException,
                        RelationServiceNotRegisteredException,
                        NoSuchMethodException,
                        InvalidRelationIdException,
                        InstanceNotFoundException,
                        InvalidRelationServiceException,
                        RelationTypeNotFoundException,
                        RoleNotFoundException,
                        InvalidRoleValueException ;

	/**
	 * Adds given object as a relation type. The object is expected to
	 * implement the RelationType interface.
	 *
	 * @param theRelTypeObj - relation type object (implementing the
	 * 				RelationType interface)
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws InvalidRelationTypeException - if there is already a relation
	 * 				type with that name
	 */
	public void addRelationType(RelationType theRelTypeObj)
                     throws IllegalArgumentException,
                            InvalidRelationTypeException ;

	/**
	 * Checks if given Role can be read in a relation of the given type.
	 *
	 * @param theRoleName - name of role to be checked
	 *
	 * @param theRelTypeName - name of the relation type
	 *
	 * @return an Integer wrapping an integer corresponding to possible
	 * 				problems represented as constants in RoleUnresolved:
	 *           	- 0 if role can be read
	 *           	- integer corresponding to RoleStatus.NO_ROLE_WITH_NAME
	 *           	- integer corresponding to RoleStatus.ROLE_NOT_READABLE
	 *
	 * @throws IllegalArgument - if null parameter
	 *
	 * @throws RelationTypeNotFoundException - if the relation type is not
	 * 				known in the Relation Service
	 */
	public Integer checkRoleReading(String theRoleName,
                                          String theRelTypeName)
                                   throws IllegalArgumentException,
                                          RelationTypeNotFoundException ;

	/**
	 * Checks if given Role can be set in a relation of given type.
	 *
	 * @param theRole - role to be checked
	 *
	 * @param theRelTypeName - name of relation type
	 *
	 * @param theInitFlg - flag to specify that the checking is done for the
	 * 				initialisation of a role, write access shall not be verified.
	 *
	 * @return
	 *       an Integer wrapping an integer corresponding to possible problems represented as
	 *       constants in RoleUnresolved:
	 *       - 0 if role can be set
	 *       - integer corresponding to RoleStatus.NO_ROLE_WITH_NAME
	 *       - integer for RoleStatus.ROLE_NOT_WRITABLE
	 *       - integer for RoleStatus.LESS_THAN_MIN_ROLE_DEGREE
	 *       - integer for RoleStatus.MORE_THAN_MAX_ROLE_DEGREE
	 *       - integer for RoleStatus.REF_MBEAN_OF_INCORRECT_CLASS
	 *       - integer for RoleStatus.REF_MBEAN_NOT_REGISTERED
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationTypeNotFoundException - if unknown relation type
	 */
	public Integer checkRoleWriting(Role theRole,
                                          String theRelTypeName,
                                          Boolean theInitFlg)
                                   throws IllegalArgumentException,
                                          RelationTypeNotFoundException ;

	/**
	 * Creates a simple relation (represented by a RelationSupport object) of
	 * given relation type, and adds it in the Relation Service.
	 * Roles are initialized according to the role list provided in parameter.
	 * The ones not initialised using that mean are set to an empty ArrayList
	 * of ObjectNames. A RelationNotification, with type
	 * RELATION_BASIC_CREATION, is sent.
	 *
	 * @param theRelId - relation identifier, to uniquely identify the relation
	 * 				inside the Relation Service
	 *
	 * @param theRelTypeName - name of the relation type (has to be created in
	 * 				the Relation Service)
	 *
	 * @param theRoleList - role list to initialise roles of the relation (can be null).
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws java.lang.IllegalArgumentException - if null paramater
	 *
	 * @throws RoleNotFoundException - if a value is provided for a role that
	 * 				does not exist in the relation type
	 *
	 * @throws InvalidRelationIdException - if relation id already used
	 *
	 * @throws RelationTypeNotFoundException - if relation type not known in
	 * 				Relation Service
	 *
	 * @throws InvalidRoleValueException - if:
	 * 				- the same role name is used for two different roles
	 *           	- the number of referenced MBeans in given value is less
	 * 					than expected minimum degree
	 *           	- the number of referenced MBeans in provided value exceeds
	 * 					expected maximum degree
	 *           	- one referenced MBean in the value is not an Object of the
	 * 					MBean class expected for that role
	 *           - a MBean provided for that role does not exist
	 */
	public void createRelation(String theRelId,
                           String theRelTypeName,
                           RoleList theRoleList)
                    throws RelationServiceNotRegisteredException,
                           IllegalArgumentException,
                           RoleNotFoundException,
                           InvalidRelationIdException,
                           RelationTypeNotFoundException,
                           InvalidRoleValueException ;

	/**
	 * Creates a relation type (RelationTypeSupport object) with given role
	 * infos (provided by the RoleInfo objects), and adds it in the Relation Service.
	 *
	 * @param theRelTypeName - name of the relation type
	 *
	 * @param theRoleInfoArray - array of role infos
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws InvalidRelationTypeException - If:
	 * 				- there is already a relation type with that name
	 *       		- the same name has been used for two different role infos
	 *       		- no role info provided
	 *       		- one null role info provided
	 */
	public void createRelationType(String theRelTypeName,
                               RoleInfo[] theRoleInfoArray)
                        throws IllegalArgumentException,
                               InvalidRelationTypeException ;

	/**
	 * Retrieves the MBeans associated to given one in a relation.
	 * This corresponds to CIM Associators and AssociatorNames operations.
	 *
	 * @param theMBeanName - ObjectName of MBean
	 *
	 * @param theRelTypeName - can be null; if specified, only the relations of
	 * 				that type will be considered in the search. Else all
	 * 				relation types are considered.
	 *
	 * @param theRoleName - can be null; if specified, only the relations where
	 * 				the MBean is referenced in that role will be considered.
	 * 				Else all roles are considered.
	 *
	 * @return an HashMap, where the keys are the ObjectNames of the MBeans
	 * 				associated to given MBean, and the value is, for each key,
	 * 				an ArrayList of the relation ids of the relations where the
	 * 				key MBean is associated to given one (as they can be
	 * 				associated in several different relations).
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 */
	public Map findAssociatedMBeans(ObjectName theMBeanName,
                                          String theRelTypeName,
                                          String theRoleName)
                                   throws IllegalArgumentException ;

	/**
	 * Retrieves the relations where a given MBean is referenced.
	 * This corresponds to the CIM "References" and "ReferenceNames" operations.
	 *
	 * @param theMBeanName - ObjectName of MBean
	 *
	 * @param theRelTypeName - can be null; if specified, only the relations of
	 * 				that type will be considered in the search. Else all
	 * 				relation types are considered.
	 *
	 * @param theRoleName - can be null; if specified, only the relations where
	 * 				the MBean is referenced in that role will be returned.
	 * 				Else all roles are considered.
	 *
	 * @return an HashMap, where the keys are the relation ids of the relations
	 * 				where the MBean is referenced, and the value is, for each
	 * 				key, an ArrayList of role names (as a MBean can be
	 * 				referenced in several roles in the same relation).
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 */
	public Map findReferencingRelations(ObjectName theMBeanName,
                                              String theRelTypeName,
                                              String theRoleName)
                                       throws IllegalArgumentException ;


	/**
	 * Returns the relation ids for relations of the given type.
	 *
	 * @param theRelTypeName - relation type name
	 *
	 * @return an ArrayList of relation ids.
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationTypeNotFoundException - if there is no relation type
	 * 				with that name.
	 */
    public List findRelationsOfType(String theRelTypeName)
                                   throws IllegalArgumentException,
                                          RelationTypeNotFoundException ;

	/**
	 * Returns all the relation ids for all the relations handled by the
	 * Relation Service
	 *
	 * @return ArrayList of String
	 */
	public List getAllRelationIds();

	/**
	 * Retrieves names of all known relation types
	 *
	 * @return ArrayList of relation type names (Strings)
	 */
	public List getAllRelationTypeNames() ;

	/**
	 * Returns all roles present in the relation
	 *
	 * @param theRelId - relation id
	 *
	 * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully retrieved) and a RoleUnresolvedList
	 * 				(for roles not readable).
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws  RelationNotFoundException - if no relation for given id
	 *
	 * @throws  RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 */
	public RoleResult getAllRoles(String theRelId)
                       throws IllegalArgumentException,
                              RelationNotFoundException,
                              RelationServiceNotRegisteredException ;

	/**
	 * Returns the flag to indicate  when a notification is received for the
	 * unregistration of a MBean referenced in a relation, if an immediate
	 * "purge" of the relations (look for the relations no longer valid) has
	 * to be performed , or if that will be performed only when the
	 * purgeRelations method will be explicitly called. true is immediate purge.
	 */
	public boolean getPurgeFlag();


	/**
	 * Retrieves MBeans referenced in the various roles of the relation.
	 *
	 * @param theRelId - relation id
	 *
	 * @return a HashMap mapping
	 * 				ObjectName -> ArrayList of String (role names)
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - if no relation for given relation id
	 */
	public Map getReferencedMBeans(String theRelId)
                                  throws IllegalArgumentException,
                                         RelationNotFoundException ;

	/**
	 * Returns name of associated relation type for given relation.
	 *
	 * @param theRelId - relation id
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - if no relation for given relation id
	 */
	public String getRelationTypeName(String theRelId)
                                     throws IllegalArgumentException,
                                            RelationNotFoundException ;

	/**
	 * Retrieves role value for given role name in given relation.
	 *
	 * @param theRelId - relation id
	 *
	 * @param theRoleName - name of role
	 *
	 * @return the ArrayList of ObjectName objects being the role value
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - if no relation with given id
	 *
	 * @throws RoleNotFoundException - if:
	 *         		- there is no role with given name
	 *            	or
	 *         		- the role is not readable.
	 */
	public List getRole(String theRelId,
                              String theRoleName)
                       throws RelationServiceNotRegisteredException,
                              IllegalArgumentException,
                              RelationNotFoundException,
                              RoleNotFoundException ;

	/**
	 * Retrieves the number of MBeans currently referenced in the given role.
	 *
	 * @param theRelId - relation id
	 *
	 * @param theRoleName - name of role
	 *
	 * @return the number of currently referenced MBeans in that role
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - if no relation with given id
	 *
	 * @throws RoleNotFoundException - if there is no role with given name
	 */
	public Integer getRoleCardinality(String theRelId,
                                            String theRoleName)
                                     throws IllegalArgumentException,
                                            RelationNotFoundException,
                                            RoleNotFoundException ;

	/**
	 * Retrieves role info for given role of a given relation type
	 *
	 * @param theRelTypeName - name of relation type
	 *
	 * @param theRoleInfoName - name of role
	 *
	 * @return RoleInfo object.
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationTypeNotFoundException - if the relation type is not
	 * 				known in the Relation Service
	 *
	 * @throws RoleInfoNotFoundException - if the role is not part of the relation type.
	 */
	public RoleInfo getRoleInfo(String theRelTypeName,
                            String theRoleInfoName)
                     throws IllegalArgumentException,
                            RelationTypeNotFoundException,
                            RoleInfoNotFoundException ;

	/**
	 * Retrieves list of role infos (RoleInfo objects) of a given relation type
	 *
	 * @param theRelTypeName - name of relation type
	 *
	 * @return ArrayList of RoleInfo.
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationTypeNotFoundException - if there is no relation type
	 * 				with that name.
	 */
	public List getRoleInfos(String theRelTypeName)
                            throws IllegalArgumentException,
                                   RelationTypeNotFoundException ;
	/**
	 * Retrieves values of roles with given names in given relation.
	 *
	 * @param theRelId - relation id
	 *
	 * @param theRoleNameArray - array of names of roles to be retrieved
	 *
	 * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully retrieved) and a RoleUnresolvedList
	 * 				(for roles not retrieved).
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws  RelationNotFoundException - if no relation with given id
	 */
	public RoleResult getRoles(String theRelId,
                           String[] theRoleNameArray)
                    throws RelationServiceNotRegisteredException,
                           IllegalArgumentException,
                           RelationNotFoundException ;

	/**
	 * Checks if there is a relation identified in Relation Service with
	 * given relation id.
	 *
	 * @param theRelId - relation id identifying the relation
	 *
	 * @return boolean: true if there is a relation, false else
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 */
	public Boolean hasRelation(String theRelId)
                              throws IllegalArgumentException ;

	/**
	 * Checks if the Relation Service is active. Current condition is that
	 * the Relation Service must be registered in the MBean Server
	 *
	 * @throws RelationServiceNotRegisteredException - if it is not registered
	 */
	public void isActive()
              throws RelationServiceNotRegisteredException ;

	/**
	 * Returns the relation id associated to the given ObjectName if the MBean
	 * has been added as a relation in the Relation Service.
	 *
	 * @param theObjName - ObjectName of supposed relation
	 *
	 * @return relation id (String) or null (if the ObjectName is not a
	 * 				relation handled by the Relation Service)
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 */
	public String isRelation(ObjectName theObjName)
                            throws IllegalArgumentException ;

	/**
	 * If the relation is represented by a MBean (created by the user and added
	 * as a relation in the Relation Service), returns the ObjectName of the MBean.
	 *
	 * @param theRelId - relation id identifying the relation
	 *
	 * @return ObjectName of the corresponding relation MBean, or null if
	 * 				the relation is not a MBean.
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - there is no relation associated to that id
	 */
	public ObjectName isRelationMBean(String theRelId)
                           throws IllegalArgumentException,
                                  RelationNotFoundException ;

	/**
	 * Purges the relations.
	 * Depending on the purgeFlag value, this method is either called
	 * automatically when a notification is received for the unregistration of
	 * a MBean referenced in a relation (if the flag is set to true), or not
	 * (if the flag is set to false). In that case it is up to the user to call
	 * it to maintain the consistency of the relations. To be kept in mind that
	 * if a MBean is unregistered and the purge not done immediately, if the
	 * ObjectName is reused and assigned to another MBean referenced in a
	 * relation, calling manually this purgeRelations() method will cause
	 * trouble, as will consider the ObjectName as corresponding to the
	 * unregistered MBean, not seeing the new one. The behavior depends on the
	 * cardinality of the role where the unregistered MBean is referenced:
	 * if removing one MBean reference in the role makes its number of
	 * references less than the minimum degree, the relation has to be removed.
	 * - if the remaining number of references after removing the MBean
	 * reference is still in the cardinality range, keep the relation and
	 * update it calling its handleMBeanUnregistration() callback.
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server.
	 */
	public void purgeRelations()
                    throws RelationServiceNotRegisteredException ;



	/**
	 * Removes given relation from the Relation Service.
	 * A RelationNotification notification is sent, its type being:
	 * 				- RelationNotification.RELATION_BASIC_REMOVAL if the
	 * 					relation was only internal to the Relation Service
	 *      		- RelationNotification.RELATION_MBEAN_REMOVAL if the
	 * 					relation is registered as a MBean.
	 *        		For MBeans referenced in such relation, nothing will be done.
	 *
	 * @param theRelId - relation id of the relation to be removed
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - if no relation corresponding to
	 * 				given relation id
	 */
	public void removeRelation(String theRelId)
                    throws RelationServiceNotRegisteredException,
                           IllegalArgumentException,
                           RelationNotFoundException ;

	/**
	 * Removes given relation type from Relation Service.
	 * The relation objects of that type will be removed from the Relation Service.
	 *
	 * @param theRelTypeName - name of the relation type to be removed
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationTypeNotFoundException - If there is no relation type
	 * 				with that name
	 */
	public void removeRelationType(String theRelTypeName)
                        throws RelationServiceNotRegisteredException,
                               IllegalArgumentException,
                               RelationTypeNotFoundException ;


	/**
	 * Sends a notification (RelationNotification) for a relation creation.
	 * The notification type is:
	 *  			- RelationNotification.RELATION_BASIC_CREATION if the
	 * 					relation is an object internal to the Relation Service
	 *  			- RelationNotification.RELATION_MBEAN_CREATION if the
	 * 					relation is a MBean added as a relation.
	 *    			The source object is the Relation Service itself. It is called
	 * 				in Relation Service createRelation() and addRelation() methods.
	 *
	 * @param theRelId - relation identifier of the updated relation
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - if there is no relation for given relation id
	 */
	public void sendRelationCreationNotification(String theRelId)
                                      throws IllegalArgumentException,
                                             RelationNotFoundException ;

	/**
	 * Sends a notification (RelationNotification) for a relation removal. The notification type is:
	 * 				- RelationNotification.RELATION_BASIC_REMOVAL if the
	 * 					relation is an object internal to the Relation Service
	 *    			- RelationNotification.RELATION_MBEAN_REMOVAL if the
	 * 					relation is a MBean added as a relation.
	 *      		The source object is the Relation Service itself.
	 *      		It is called in Relation Service removeRelation() method.
	 *
	 * @param theRelId - relation identifier of the updated relation
	 *
	 * @param theUnregMBeanList - ArrayList of ObjectNames of MBeans expected
	 * 				to be unregistered due to relation removal (can be null)
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - if there is no relation for given relation id
	 */
	public void sendRelationRemovalNotification(String theRelId,
                                            List theUnregMBeanList)
                                     throws IllegalArgumentException,
                                            RelationNotFoundException ;

	/**
	 * Sends a notification (RelationNotification) for a role update in the
	 * given relation. The notification type is:
	 * 				- RelationNotification.RELATION_BASIC_UPDATE if the
	 * 					relation is an object internal to the Relation Service
	 *				- RelationNotification.RELATION_MBEAN_UPDATE if the relation
	 * 					is a MBean added as a relation.
	 * 				The source object is the Relation Service itself.
	 * 				It is called in relation MBean setRole() (for given role)
	 * 				and setRoles() (for each role) methods (implementation
	 * 				provided in RelationSupport class).
	 * 				It is also called in Relation Service setRole() (for given
	 * 				role) and setRoles() (for each role) methods.
	 *
	 * @param theRelId - relation identifier of the updated relation
	 *
	 * @param theNewRole - new role (name and new value)
	 *
	 * @param theOldRoleValue - old role value (ArrayList of ObjectName objects)
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - if there is no relation for given relation id
	 */
	public void sendRoleUpdateNotification(String theRelId,
                                       Role theNewRole,
                                       List theOldRoleValue)
                                throws IllegalArgumentException,
                                       RelationNotFoundException ;

	/**
	 * Sets the flag to indicate if when a notification is received for the
	 * unregistration of a MBean referenced in a relation, if an immediate
	 * "purge" of the relations (look for the relations no longer valid) has to
	 * be performed , or if that will be performed only when the purgeRelations
	 * method will be explicitly called. true is immediate purge.
	 *
	 * @param thePurgeFlg - flag
	 */
	public void setPurgeFlag(boolean thePurgeFlg) ;


	/**
	 * Sets the given role in given relation.
	 * Will check the role according to its corresponding role definition
	 * provided in relation's relation type.
	 * The Relation Service will keep track of the change to keep the
	 * consistency of relations by handling referenced MBean unregistrations.
	 *
	 * @param theRelId - relation id
	 *
	 * @param theRole - role to be set (name and new value)
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - if no relation with given id
	 *
	 * @throws RoleNotFoundException - if:
	 *           	- internal relation
	 *           		and
	 *           	- the role does not exist or is not writable
	 *
	 * @throws InvalidRoleValueException - if internal relation and value
	 * 				provided for role is not valid:
	 *           	- the number of referenced MBeans in given value is less
	 * 					than expected minimum degree
	 *           	or
	 *           	- the number of referenced MBeans in provided value exceeds
	 * 					expected maximum degree
	 *           	or
	 *           	- one referenced MBean in the value is not an Object of the
	 * 					MBean class expected for that role
	 *           	or
	 *           	- a MBean provided for that role does not exist
	 *
	 * @throws  RelationTypeNotFoundException - if unknown relation type
	 */
	public void setRole(String theRelId,
                    Role theRole)
             throws RelationServiceNotRegisteredException,
                    IllegalArgumentException,
                    RelationNotFoundException,
                    RoleNotFoundException,
                    InvalidRoleValueException,
                    RelationTypeNotFoundException;


	/**
	 * Sets the given roles in given relation.
	 * Will check the role according to its corresponding role definition
	 * provided in relation's relation type.
	 * The Relation Service keeps track of the changes to keep the consistency
	 * of relations by handling referenced MBean unregistrations.
	 *
	 * @param theRelId - relation id
	 *
	 * @param theRoleList - list of roles to be set
	 *
	 * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully set) and a RoleUnresolvedList (for roles not set).
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationNotFoundException - if no relation with given id
	 */
	public RoleResult setRoles(String theRelId,
                           RoleList theRoleList)
                    throws RelationServiceNotRegisteredException,
                           IllegalArgumentException,
                           RelationNotFoundException ;

	/**
	 * Handles update of the Relation Service role map for the update of given
	 * role in given relation. It is called in relation MBean setRole() (for
	 * given role) and setRoles() (for each role) methods (implementation
	 * provided in RelationSupport class).
	 * It is also called in Relation Service setRole() (for given role) and
	 * setRoles() (for each role) methods.
	 * To allow the Relation Service to maintain the consistency (in case of
	 * MBean unregistration) and to be able to perform queries, this method
	 * must be called when a role is updated.
	 *
	 * @param theRelId - relation identifier of the updated relation
	 *
	 * @param theNewRole - new role (name and new value)
	 *
	 * @param theOldRoleValue - old role value (ArrayList of ObjectName objects)
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @throws RelationNotFoundException - if no relation for given id.
	 */
	public void updateRoleMap(String theRelId,
                          Role theNewRole,
                          List theOldRoleValue)
                   throws IllegalArgumentException,
                          RelationServiceNotRegisteredException,
                          RelationNotFoundException ;
}