/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.util.Map;
import java.util.List;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;

import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanRegistration;
import javax.management.MBeanException;
import javax.management.ReflectionException;
import javax.management.InstanceNotFoundException;

/**
 * <P>A RelationSupport object is used internally by the Relation Service to
 * represent simple relations (only roles, no properties or methods), with an
 * unlimited number of roles, of any relation type. As internal representation,
 * it is not exposed to the user.
 * <P>
 * RelationSupport class conforms to the design patterns of standard MBean.
 * So the user can decide to instantiate a RelationSupport object himself as a
 * MBean (as it follows the MBean design patterns), to register it in the
 * MBean Server, and then to add it in the Relation Service.
 * <P>
 * The user can also, when creating his own MBean relation class, have it
 * extending RelationSupport, to retrieve the implementations of required
 * interfaces (see below).
 * <P>
 * It is also possible to have in a user relation MBean class a member being
 * a RelationSupport object, and to implement the required interfaces by
 * delegating all to this member.
 * <P>
 * RelationSupport implements the Relation interface (to be handled by the
 * Relation Service).
 * <P>
 * It implements also the MBeanRegistration interface to be able to retrieve
 * the MBean Server where it is registered (if registered as a MBean) to
 * access to its Relation Service.
 */
public class RelationSupport implements RelationSupportMBean, MBeanRegistration
{
    // Relation identifier (expected to be unique in the Relation Service
    // where the RelationSupport object will be added)
	String relId = null;

    // Relation type name (must be known in the Relation Service where the
    // relation will be added)
	String relTypeName = null;

	RoleList roleList = null;
	ObjectName objectName = null;

    // ObjectName of the Relation Service where the relation will be added
    // REQUIRED if the RelationSupport is created by the user to be registered
	// as a MBean, as it will have to access the Relation Service via the MBean
    // Server to perform the check regarding the relation type.
    // Is null if current object is directly created by the Relation Service,
    // as the object will directly access it.
	ObjectName relServiceName = null;

    // Reference to the MBean Server where the Relation Service is
    // registered
    // REQUIRED if the RelationSupport is created by the user to be registered
    // as a MBean, as it will have to access the Relation Service via the MBean
    // Server to perform the check regarding the relation type.
    // If the Relationbase object is created by the Relation Service (use of
    // createRelation() method), this is null as not needed, direct access to
    // the Relation Service.
    // If the Relationbase object is created by the user and registered as a
    // MBean, this is set by the preRegister() method below.
	MBeanServer mbeanServer = null;

	Boolean isInRelation = new Boolean(false);

	private HashMap roleToValueTable = new HashMap();


	/**
	 * Creates object.
	 * <P>This constructor has to be used when the user relation MBean implements
	 * the interfaces expected to be supported by a relation by delegating to a
	 * RelationSupport object. This object needs to know the Relation Service
	 * expected to handle the relation. So it has to know the MBean Server
	 * where the Relation Service is registered.
	 * <P>
	 * According to a limitation, a relation MBean must be registered in the
	 * same MBean Server as the Relation Service expected to handle it. So the
	 * user relation MBean has to be created and registered, and then the
	 * wrapped RelationSupport object can be created with identified MBean Server.
	 * <P>
	 * Nothing is done at the Relation Service level, i.e. the RelationSupport
	 * object is not added, and no check if the provided values are correct.
	 * The object is always created, EXCEPT if:
	 * <P>			- one required parameter is not provided
	 * <P> 			- the same name is used for two roles.
	 * <P>
	 * To be handled as a relation, the object has then to be added in the
	 * Relation Service using the Relation Service method addRelation().
	 * <P>
	 * @param theRelId - relation identifier, to identify the relation in
	 * 				the Relation Service. Expected to be unique in the given
	 * 				Relation Service.
	 *
	 * @param theRelServiceName - ObjectName of the Relation Service where
	 * 				the relation will be registered. It is required as this is
	 * 				the Relation Service that is aware of the definition of
	 * 				the relation type of given relation, so that will be able
	 * 				to check update operations (set).
	 *
	 * @param theRelServiceMBeanServer - MBean Server where the wrapping MBean
	 * 				is or will be registered. Expected to be the MBean Server
	 * 				where the Relation Service is or will be registered.
	 *
	 * @param  theRelTypeName - Name of relation type.
	 *         		Expected to have been created in given Relation Service.
	 *
	 * @param theRoleList - list of roles (Role objects) to initialised the
	 * 				relation. Can be null. Expected to conform to relation info
	 * 				in associated relation type.
	 *
	 * @exception InvalidRoleValueException - if the same name is used for two roles.
	 *
	 * @exception java.lang.IllegalArgumentException - if a required value
	 * 				(Relation Service Object Name, etc.) is not provided as parameter.
	 */
	public RelationSupport(String theRelId,
                       ObjectName theRelServiceName,
                       MBeanServer theRelServiceMBeanServer,
                       String theRelTypeName,
                       RoleList theRoleList)
                throws InvalidRoleValueException,
					   IllegalArgumentException
	{
		if(theRelServiceName == null || theRelId == null || theRelTypeName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		if(theRoleList == null)
			throw new InvalidRoleValueException("null RoleList is not possible!!!");

		if(this.isDuplicateRolePresent(theRoleList))
		    throw new InvalidRoleValueException(RelationErrs.ERR_INVALID_ROLE);

		initRoleToValueTable(theRoleList);
		this.relId = theRelId;
		this.relServiceName = theRelServiceName;
		this.mbeanServer = theRelServiceMBeanServer;
		this.relTypeName = theRelTypeName;
		this.roleList = theRoleList;
	}

	/**
	 * Creates object.
	 * <P>This constructor has to be used when the RelationSupport object will
	 * be registered as a MBean by the user, or when creating a user relation
	 * MBean those class extends RelationSupport. Nothing is done at the
	 * Relation Service level, i.e. the RelationSupport object is not added,
	 * and no check if the provided values are correct. The object is always
	 * created, EXCEPT if:
	 * <P>			- one mandatory parameter is not provided
	 * <P>     		- the same name is used for two roles.
	 * <P>
	 * To be handled as a relation, the object has then to be added in the
	 * Relation Service using Relation Service method addRelation().
	 *
	 * @param theRelId - relation identifier, to identify the relation in the
	 * 				Relation Service. Expected to be unique in the given
	 * 				Relation Service.
	 *
	 * @param theRelServiceName - ObjectName of the Relation Service where the
	 * 				relation will be registered. It is required as this is the
	 * 				Relation Service that is aware of the definition of the
	 * 				relation type of given relation, so that will be able to
	 * 				check update operations (set).
	 *
	 * @param theRelTypeName - Name of relation type.
	 *           	Expected to have been created in given Relation Service.
	 *
	 * @param theRoleList - list of roles (Role objects) to initialised the
	 * 				relation. Can be null. Expected to conform to relation info
	 * 				in associated relation type.
	 *
	 * @exception InvalidRoleValueException - if the same name is used for two roles.
	 *
	 * @exception java.lang.IllegalArgumentException - if a required value
	 * 				(Relation Service Object Name, etc.) is not provided as parameter.
	 */
	public RelationSupport(String theRelId,
                       ObjectName theRelServiceName,
                       String theRelTypeName,
                       RoleList theRoleList)
                throws InvalidRoleValueException,
					   IllegalArgumentException
	{
		if(theRelId == null || theRelServiceName == null || theRelTypeName == null)
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		if(this.isDuplicateRolePresent(theRoleList))
		    throw new InvalidRoleValueException(RelationErrs.ERR_INVALID_ROLE);

		initRoleToValueTable(theRoleList);
		this.relId = theRelId;
		this.relServiceName = theRelServiceName;
		this.relTypeName = theRelTypeName;
		this.roleList = theRoleList;
	}

	/**
	 * Returns all roles present in the relation
	 *
	 * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully retrieved) and a RoleUnresolvedList
	 * 				(for roles not readable).
	 *
	 * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 */
	public RoleResult getAllRoles() throws RelationServiceNotRegisteredException
	{
        if(! (mbeanServer.isRegistered(this.relServiceName)) )
	          throw new RelationServiceNotRegisteredException(
			  						RelationErrs.ERR_REL_SERV_NOT_FOUND);

		RoleResult result = null;

		// Get the RoleResult from a local friend method. This call does not
		// occur from RelationService and the relationService will also be null
		result = getAllRolesFriend(null);

		return result;
	}

	/**
	 * Retrieves MBeans referenced in the various roles of the relation.
	 *
	 * @return  a HashMap mapping:
	 *           ObjectName -> ArrayList of String (role names)
	 */
	public Map getReferencedMBeans()
	{
		HashMap refMBeanMap = new HashMap();

		//Easiest way is to get the RoleValues from the local table and form a map.
		Iterator iter1 = roleToValueTable.values().iterator();

		while(iter1.hasNext())
		{
			Role role = (Role)(iter1.next());
			String roleName = role.getRoleName();

			//retrieves the ObjectNames of the MBean from RoleValue
			ArrayList refMBeanList = (ArrayList)(role.getRoleValue());

			Iterator iter2 = refMBeanList.iterator();
			while(iter2.hasNext())
			{
				ObjectName mbeanName = (ObjectName)(iter2.next());

				ArrayList list = (ArrayList)(refMBeanMap.get(mbeanName));
				boolean flag = false;
				if(list == null )
				{
					list = new ArrayList();
					flag = true;
				}
				list.add(roleName);
				if(flag)
				{
					refMBeanMap.put(mbeanName,list);
				}
			}
		}

		return refMBeanMap;
	}

	/**
	 * Returns relation identifier (used to uniquely identify the relation
	 * inside the Relation Service).
	 *
	 * @return  Relation identifier
	 */
	public String getRelationId()
	{
		return this.relId;
	}

	/**
	 * Returns ObjectName of the Relation Service handling the relation
	 *
	 * @return  ObjectName of the Relation Service handling the relation
	 */
	public ObjectName getRelationServiceName()
	{
		return this.relServiceName;
	}

	/**
	 * Returns name of associated relation type.
	 *
	 * @return The name of associated relation type
	 */
	public String getRelationTypeName()
	{
		return this.relTypeName;
	}

	/**
	 * Retrieves role value for given role name.
	 * <P>Checks if the role exists and is readable according to the relation type.
	 *
	 * @param theRoleName - name of role
	 *
	 * @return the ArrayList of ObjectName objects being the role value
	 *
	 * @exception java.lang.IllegalArgumentException - if null role name
	 *
	 * @exception RoleNotFoundException - if:
	 * <BR/>		- there is no role with given name
	 * <BR/>		- the role is not readable.
	 *
	 * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 */
	public List getRole(String theRoleName)
                       throws IllegalArgumentException,
                              RoleNotFoundException,
							  RelationServiceNotRegisteredException
	{
        if(theRoleName == null)
            throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

        if(! (mbeanServer.isRegistered(this.relServiceName)) )
            throw new RelationServiceNotRegisteredException(
										RelationErrs.ERR_REL_SERV_NOT_FOUND);

        ArrayList toRet = null;
    	toRet = (ArrayList)(getRoleFriend(theRoleName,false,null));

		return toRet;
	}

	/**
	 * Returns the number of MBeans currently referenced in the given role
	 *
	 * @param theRoleName - name of role
	 *
	 * @return the number of currently referenced MBeans in that role
	 *
	 * @exception java.lang.IllegalArgumentException - if null role name
	 *
	 * @exception RoleNotFoundException - if there is no role with given name
	 */
	public Integer getRoleCardinality(String theRoleName)
								throws IllegalArgumentException,
									   RoleNotFoundException
	{
		if(theRoleName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Role role = (Role)(roleToValueTable.get(theRoleName));

		if(role == null)
		{
			throw new RoleNotFoundException();
		}

		ArrayList roleValue = (ArrayList)(role.getRoleValue());

		return new Integer(roleValue.size());
	}

	/**
	 * Retrieves values of roles with given names.
	 * <P>Checks for each role if it exists and is readable according to
	 * the relation type.
	 *
	 * @param theRoleNameArray - array of names of roles to be retrieved
	 *
	 * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully retrieved) and a RoleUnresolvedList
	 * 				(for roles not retrieved).
	 *
	 * @exception java.lang.IllegalArgumentException - if null role name
	 *
	 * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 */
	public RoleResult getRoles(String[] theRoleNameArray)
                    	throws IllegalArgumentException,
						   		RelationServiceNotRegisteredException
	{
		if(theRoleNameArray == null)
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		if( !( mbeanServer.isRegistered(this.relServiceName) ) )
			throw new RelationServiceNotRegisteredException();

		return getRolesFriend(theRoleNameArray,null);
	}

	/**
	 * Callback used by the Relation Service when a MBean referenced in a role
	 * is unregistered.
	 * <P>The Relation Service will call this method to let the relation take
	 * action to reflect the impact of such unregistration.
	 * BEWARE. the user is not expected to call this method.
	 * <P>
	 * Current implementation is to set the role with its current value
	 * (list of ObjectNames of referenced MBeans) without the unregistered one.
	 *
	 * @param theObjName - ObjectName of unregistered MBean
	 *
	 * @param theRoleName - name of role where the MBean is referenced
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 *
	 * @exception RoleNotFoundException - if role does not exist in
	 * 				the relation or is not writable
	 *
	 * @exception InvalidRoleValueException - if role value does not conform to
	 * 				the associated role info (this will never happen when
	 * 				called from the Relation Service)
	 *
	 * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 *
	 * @exception RelationTypeNotFoundException - if the relation type has not
	 * 				been declared in the Relation Service.
	 *
	 * @exception RelationNotFoundException - if this method is called for a
	 * 				relation MBean not added in the Relation Service.
	 */
	public void handleMBeanUnregistration(ObjectName theObjName,
                                      String theRoleName)
                               throws IllegalArgumentException,
                                      RoleNotFoundException,
                                      InvalidRoleValueException,
                                      RelationServiceNotRegisteredException,
                                      RelationTypeNotFoundException,
									  RelationNotFoundException
	{
		if(theObjName == null || theRoleName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Role role = (Role)(roleToValueTable.get(theRoleName));

		if(role == null)
		{
			throw new RoleNotFoundException();
		}

		ArrayList roleValue = (ArrayList)(role.getRoleValue());
		//Remove the UnregisteredMBean Name from the RoleValue and set it.
		//Use clone..that would be better.
		ArrayList toSet = (ArrayList)(roleValue.clone());
		toSet.remove(theObjName);
		//construct a new Role.
		Role toSetRole = new Role(theRoleName,toSet);
		setRoleFriend(toSetRole,false,null);
	}

	/**
	 * Returns an internal flag specifying if the object is still handled by
	 * the Relation Service.
	 *
	 * @return Internal flag specifying if the object is still handled by
	 * 				the Relation Service.
	 * Specified by:
	 *    isInRelationService in interface RelationSupportMBean
	 */
	public Boolean isInRelationService()
	{
		return this.isInRelation;
	}

	/**
	 * Description copied from interface: MBeanRegistration.
	 * <P>
	 * Allows the MBean to perform any operations needed after having been
	 * de-registered in the MBean server.
	 * <P>
	 * Specified by:
	 *           postDeregister in interface MBeanRegistration
	 */
	public void postDeregister()
	{
		//Method to satisfy MBeanRegistration Interface.. Probably can be used later
	}

	/**
	 * Description copied from interface: MBeanRegistration.
	 * <P>
	 * Allows the MBean to perform any operations needed after having been
	 * registered in the MBean server or after the registration has failed.
	 * <P>
	 * Specified by:
	 *     postRegister in interface MBeanRegistration
	 *     Tags copied from interface: MBeanRegistration
	 *
	 * @param  registrationDone - Indicates whether or not the MBean has been
	 * 				successfully registered in the MBean server. The value
	 * 				false means that the registration phase has failed
	 */
	public void postRegister(Boolean registrationDone)
	{
		//Method to satisfy MBeanRegistration Interface.. Probably can be used later
	}

    /**
     * Description copied from interface: MBeanRegistration.
     * <P>
     * Allows the MBean to perform any operations it needs before being
	 * de-registered by the MBean server.
	 * <P>
     * Specified by:
     *      preDeregister in interface MBeanRegistration
     *      Tags copied from interface: MBeanRegistration
     *
     * @exception java.langException - This exception should be caught by the
	 * 				MBean server and re-thrown as an MBeanRegistrationException.
     */
	public void preDeregister() throws Exception
	{
		//Method to satisfy MBeanRegistration Interface.. Probably can be used later
	}

	/**
	 * Description copied from interface: MBeanRegistration.
	 * <P>
	 * Allows the MBean to perform any operations it needs before being
	 * registered in the MBean server. If the name of the MBean is not specified,
	 * the MBean can provide a name for its registration. If any exception is
	 * raised, the MBean will not be registered in the MBean server.
	 * <P>
	 * Specified by:
	 *      preRegister in interface MBeanRegistration
	 *      Tags copied from interface: MBeanRegistration
	 *
	 * @param server - The MBean server in which the MBean will be registered.
	 *
	 * @param name - The object name of the MBean.
	 *
	 * @return The name of the MBean registered.
	 *
	 * @exception java.lang.Exception - This exception should be caught by the
	 * 				MBean server and re-thrown as an MBeanRegistrationException.
	 */
	public ObjectName preRegister(MBeanServer server, ObjectName name)
													throws Exception
	{
		this.mbeanServer = server;
		this.objectName = name;

		return name;
	}

	/**
	 * Returns all roles in the relation without checking read mode
	 *
	 * @return a RoleList
	 */
	public RoleList retrieveAllRoles()
	{
		return new RoleList(new ArrayList(roleToValueTable.values()));
	}

	/**
	 * Sets the flag to specify that it is handled or not by the Relation Service
	 * BEWARE, this method has to be exposed for the user relation MBeans, as
	 * the Relation Service will access them through their management interface.
	 * It is RECOMMENDED NOT to use this method. Using it does not affect the
	 * registration of the relation object in the Relation Service, but will
	 * provide wrong information about it!!!!
	 * <P>
	 * Specified by:
	 *        setRelationServiceManagementFlag in interface RelationSupportMBean
	 *
	 * @param theFlg If true the flag is handled by the relation service false otherwise
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 */
	public void setRelationServiceManagementFlag(Boolean theFlg)
										throws IllegalArgumentException
	{
		if(theFlg == null)
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		this.isInRelation = theFlg;
	}

	/**
	 * Sets the given role.
	 * <P>Will check the role according to its corresponding role definition
	 * provided in relation's relation type.
	 * Will send a notification (RelationNotification with type
	 * RELATION_BASIC_UPDATE or RELATION_MBEAN_UPDATE, depending if the
	 * relation is a MBean or not).
	 *
	 * @param  theRole - role to be set (name and new value)
	 *
	 * @exception java.lang.IllegalArgumentException - if null role
	 *
	 * @exception RoleNotFoundException - if the role is not writable (no test
	 * 				on the write access mode performed when initialising the role)
	 *
	 * @exception InvalidRoleValueException - if value provided for role is not valid, i.e.:
	 * 				- the number of referenced MBeans in given value is less
	 * 					than expected minimum degree
	 *           	- the number of referenced MBeans in provided value exceeds
	 * 					expected maximum degree
	 *           	- one referenced MBean in the value is not an Object of the
	 * 					MBean class expected for that role
	 *           	- a MBean provided for that role does not exist
	 *
	 * @exception RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
	 * @exception RelationTypeNotFoundException - if the relation type has not
	 * 				been declared in the Relation Service
	 *
	 * @exception RelationNotFoundException - if the relation has not been added
	 * 				in the Relation Service.
	 */
	public void setRole(Role theRole)
            			throws 	IllegalArgumentException,
								InvalidRoleValueException,
								RelationNotFoundException,
								RelationServiceNotRegisteredException,
								RelationTypeNotFoundException,
								RoleNotFoundException
	{
		//This method should actually call the RelationService's updateRoleMap().
		//That will take care of updating this Role and sending the Notification
		//Note: Only the RelationService is capable of sending
		//RelationNotifications and not any other Object.

		if (theRole == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		if( !( mbeanServer.isRegistered(this.relServiceName) ) )
			throw new RelationServiceNotRegisteredException();

		setRoleFriend(theRole,false,null);
	}

	/**
	 * Sets the given roles.
	 * <P>Will check the role according to its corresponding role definition
	 * provided in relation's relation type will send one notification
	 * (RelationNotification with type RELATION_BASIC_UPDATE or
	 * RELATION_MBEAN_UPDATE, depending if the relation is a MBean or not)
	 * per updated  role.
	 *
	 * @param theRoleList - list of roles to be set
	 *
	 * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully set) and a RoleUnresolvedList (for roles not set).
	 *
	 * @exception java.lang.IllegalArgumentException - if null role name
	 *
	 * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 *
	 * @exception RelationTypeNotFoundException - if the relation type has not
	 * 				been declared in the Relation Service.
	 *
	 * @exception  RelationNotFoundException - if the relation MBean has not
	 * 				been added in the Relation Service.
	 */
	public RoleResult setRoles(RoleList theRoleList)
                    throws IllegalArgumentException,
                           RelationServiceNotRegisteredException,
                           RelationTypeNotFoundException,
						   RelationNotFoundException
	{

		if(theRoleList == null)
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		if(! (mbeanServer.isRegistered(relServiceName)) )
			throw new RelationServiceNotRegisteredException(
									RelationErrs.ERR_REL_SERV_NOT_FOUND);

		return setRolesFriend(theRoleList, false, null);
	}


	//************************ package methods ****************************//

	void handleMBeanUnregistrationFriend(ObjectName theObjName,
                                      String theRoleName,RelationService service)
                               throws IllegalArgumentException,
                                      RoleNotFoundException,
                                      InvalidRoleValueException,
                                      RelationServiceNotRegisteredException,
                                      RelationTypeNotFoundException,
									  RelationNotFoundException
	{
		if (theObjName == null || theRoleName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Role role = (Role)(roleToValueTable.get(theRoleName));

		if (role == null) {
			throw new RoleNotFoundException();
		}

		ArrayList roleValue = (ArrayList)(role.getRoleValue());
		//Remove the UnregisteredMBean Name from the RoleValue and set it.
		//Use clone..that would be better.
		ArrayList toSet = (ArrayList)(roleValue.clone());
		toSet.remove(theObjName);
		//construct a new Role.
		Role toSetRole = new Role(theRoleName,toSet);
		setRoleFriend(toSetRole,true,service);
	}


	//*****************Methods private to RelationSupport ********************//

	private boolean isDuplicateRolePresent(RoleList list)
	{
	    boolean toRet = false;

	    if(list != null)
	    {
		    for(int i=0;i<list.size();i++)
		    {
				Role role1 = (Role)(list.get(i));
				for(int j=0;j<list.size();j++)
				{
					Role role2 = (Role)(list.get(j));
					if(i != j)
					{
						if(role1.getRoleName().trim().equals(role2.getRoleName().trim()))
						{
							toRet = true;
							break;
						}
					}
				}
				if(toRet == true)
				{
					break;
				}

			}
	 	}

		/*
		for(int i=0;i<list.size();i++)
		{
			for(int j=list.size();j<0;j--)
			{
				Role role1 = (Role)(list.get(i));
				Role role2 = (Role)(list.get(j));
				if(role1.getRoleName().trim().equals(role2.getRoleName().trim()))
				{
					toRet = true;
					break;
				}
			}
			if(toRet == true)
			{
				break;
			}
		}
		*/

	    return toRet;
	}

	private Role getRoleFromRoleList(String roleName)
	{
		for(int i=0;i<roleList.size();i++)
		{
	        Role temp = (Role)(roleList.get(i));
	        if(temp.getRoleName().equals(roleName))
			{
                return temp;
	        }
	    }

		return null;
	}

	private List getRoleValueFromRoleList(String roleName)
	{
	    for(int i=0;i<roleList.size();i++)
		{
	        Role temp = (Role)(roleList.get(i));
	        if(temp.getRoleName().equals(roleName))
			{
                return temp.getRoleValue();
	        }
	    }

	    return null;
	}

	private RoleInfo getRoleInfoFromList(ArrayList roleInfoList,String roleName)
	{
		RoleInfo toRet = null;

		for(int i=0;i<roleInfoList.size();i++)
		{
			toRet = (RoleInfo)(roleInfoList.get(i));
			if(roleName.equals(toRet.getName()))
			{
				return toRet;
			}
		}
		return toRet;
	}

	private ArrayList getRoleNamesGivenObjectName(ObjectName name)
	{
		ArrayList toRet = new ArrayList();
		for(int i=0;i<roleList.size();i++)
		{
			Role temp = (Role)(roleList.get(i));
			ArrayList roleValueList = (ArrayList)(this.getRoleValueFromRoleList(temp.getRoleName()));

			for(int j=0;j<roleValueList.size();j++)
			{
				ObjectName toComp = (ObjectName)(roleValueList.get(j));
				if(toComp.toString().equals(name.toString()))
				{
					toRet.add(temp.getRoleName());
				}
			}
		}

		return toRet;
	}

	private void print(String s)
	{
		System.out.println(s);
	}

	private void printLine()
	{
		System.out.println("_____________________________________________________________________________________________________________");
	}

	private void printLineBet()
	{
		System.out.println("\t\t\t______________________________________________________");
	}

	private void printException(Exception theExc)
	{
		theExc.printStackTrace();
		StringBuffer excName = new StringBuffer();
		Exception exc = null;

		if(theExc instanceof MBeanException)
		{
			boolean isMBeanExcFlg = true;
		    excName.append("MBeanException->");
	 	    Exception exc1 = theExc;
	  	    while(isMBeanExcFlg)
			{
				exc = ((MBeanException)exc1).getTargetException();
				if(!(exc instanceof MBeanException))
				{
					isMBeanExcFlg = false;
				}
				else
				{
				    exc1 = exc;
					excName.append("MBeanException->");
				}
		   	}
	   	}
		else
		{
			exc = theExc;
		}

		excName.append((exc.getClass()).getName());
		print("Exception: " + excName.toString());
		print(exc.getMessage());
		print("\nEXITING...\n");
		System.exit(1);
    }

	// print a list
	private void printList(ArrayList theList)
	{
   		if(theList == null)
		{
		    return;
	   	}

   		StringBuffer listDesc = new StringBuffer();

	   	for(Iterator iter = theList.iterator(); iter.hasNext();)
		{
		    listDesc.append("\t");
		    Object currObj = iter.next();
		    listDesc.append(currObj.toString());
		    if (iter.hasNext())
			{
		  		listDesc.append(",\n");
		    }
	   	}

		print(listDesc.toString());
		return;
 	}

	private void printRoleResult(RoleResult theRoleResult)
	{
   		print("\tPrinting RoleResult");
		if(theRoleResult != null)
		{
			try
			{
				RoleList roleList = theRoleResult.getRoles();
			    print("\t-> Printing the RoleList");
		    	printList(roleList);
		 	    RoleUnresolvedList roleUnresList =theRoleResult.getRolesUnresolved();
		  	    print("\t-> Printing the RoleUnresolvedList");
				printList(roleUnresList);
			}
			catch(Exception e)
			{
				print("\t!!! Could not print RoleResult!!!");
				printException(e);
			}
		}
    }

    private void initRoleToValueTable(RoleList theRoleList)
									throws InvalidRoleValueException
	{
		if(theRoleList == null)
		{
		    return;
		}

		synchronized(roleToValueTable)
		{
		    for (Iterator roleIter = theRoleList.iterator(); roleIter.hasNext();)
			{
				Role currRole = (Role)(roleIter.next());
				String currRoleName = currRole.getRoleName();

				if(roleToValueTable.containsKey(currRoleName))
				{
				    throw new InvalidRoleValueException("");
				}

				roleToValueTable.put(new String(currRoleName), (Role)(currRole.clone()));
	    	}
		}
    }

	/**
	 *	This method is a friend method to getAllRoles.
	 */
    RoleResult getAllRolesFriend(RelationService service)
    					throws RelationServiceNotRegisteredException
	{
		ArrayList roleNamesList = new ArrayList(roleToValueTable.keySet());
		String[] roleNames = new String[roleNamesList.size()];
		int i = 0;
		Iterator iter = roleNamesList.iterator();
		String roleName = null;

		while(iter.hasNext())
		{
			roleName = (String)(iter.next());
			roleNames[i] = roleName;
			i++;
		}

		RoleResult result = getRolesFriend(roleNames,service);
		return result;
    }


    /**
     *	This method returns an RoleResult provided a string[] of roleNames.
     */
    RoleResult getRolesFriend(String[] theRoleNameArray, RelationService service)
    						throws RelationServiceNotRegisteredException
	{
		if(theRoleNameArray == null )
		{
		    throw new IllegalArgumentException();
		}

		RoleList roleList = new RoleList();
		RoleUnresolvedList roleUnresList = new RoleUnresolvedList();

		for(int i = 0; i < theRoleNameArray.length; i++)
		{
		    String roleName = theRoleNameArray[i];
		    Object result = null;

			try
			{
				result = getRoleFriend(roleName,true,service);
		    }
			catch (RoleNotFoundException exc)
		    {
				return null;
		    }

			if (result instanceof Role)
		    {
			    roleList.add((Role)result);

		    }
			else if (result instanceof RoleUnresolved)
		    {
			    roleUnresList.add((RoleUnresolved)result);
		    }
		}

		RoleResult toRet = new RoleResult(roleList, roleUnresList);
		return toRet;
    }

	/**
	 *	This method gets the Role/RoleUnresolved object provided a RoleName.
	 */
    Object getRoleFriend(String theRoleName,
						 boolean theMultiRoleFlg,
						 RelationService service)
    			throws   IllegalArgumentException,
						 RoleNotFoundException,
						 RelationServiceNotRegisteredException
	{
		//This variable will indicate the problem type in the Role
		int problemType = 0;
		Role role = null;
		//First get the Role from the RoleTable.
	    role = (Role)(roleToValueTable.get(theRoleName));

		if (role == null)
		{
			problemType = RoleStatus.NO_ROLE_WITH_NAME;

		}
		else
		{

			if(service != null)
			{
				try
				{
					problemType = (service.checkRoleReading(theRoleName,relTypeName)).intValue();
				}
				catch(RelationTypeNotFoundException ee)
				{
					throw new RuntimeException(ee.getMessage());
				}
			}
			else
			{
				//Invoke to find out if the RoleReading is allowed.
				try
				{
					Object[] params = new Object[]{theRoleName,relTypeName};
					String[] sig = new String[]{"java.lang.String","java.lang.String"};

					problemType = ((Integer)(mbeanServer.invoke(relServiceName,
									"checkRoleReading",params,sig))).intValue();

				}
				catch (MBeanException me)
				{
					throw new RuntimeException(me.getMessage());
				}
				catch (ReflectionException re)
				{
					throw new RuntimeException(re.getMessage());
				}
				catch (InstanceNotFoundException ie)
				{
					throw new RelationServiceNotRegisteredException(
								    ie.getMessage());
				}
			}
		}

		Object result = null;
		if (problemType == 0)
		{
			//The role has been retrieved
		    if (!(theMultiRoleFlg))
		    {
				result = (ArrayList)(((ArrayList)(role.getRoleValue())).clone());
		    }
		    else
		    {
				result = (Role)(role.clone());
	    	}

		}
		else if(problemType != 0)
		{
	    	// Role not retrieved

		    if (!(theMultiRoleFlg))
		    {
				try
				{
					if(problemType == RoleStatus.NO_ROLE_WITH_NAME ||
							problemType == RoleStatus.ROLE_NOT_READABLE ||
							problemType == RoleStatus.ROLE_NOT_WRITABLE)
					{
                        throw new RoleNotFoundException();
					}
					else
					{
						throw new InvalidRoleValueException();
					}
				}
				catch (InvalidRoleValueException exc)
				{
				    throw new RuntimeException(exc.getMessage());
				}
			}
			else
		    {
				result = new RoleUnresolved(theRoleName, null, problemType);
		    }
		}

		return result;
    }


	/**
	 * This method is a localfriend to setRoles.
	 */
    RoleResult setRolesFriend(RoleList theRoleList,
							  boolean theRelServCallFlg,
							  RelationService theRelServ)
					throws 	  IllegalArgumentException,
							  RelationTypeNotFoundException,
							  RelationNotFoundException
	{
		if (theRoleList == null)
		{
			throw new IllegalArgumentException();
		}

		RoleList roleList = new RoleList();
		RoleUnresolvedList roleUnresList = new RoleUnresolvedList();
		Iterator iter = theRoleList.iterator();

		try
		{
			while(iter.hasNext())
			{
				Role role = (Role)(iter.next());
				Object roleResult = null;
				try
				{
					roleResult = setRoleFriend(role,theRelServCallFlg,theRelServ);
				}
				catch(Exception e)
				{
				}

				if(roleResult instanceof Role)
				{
					roleList.add((Role)roleResult);
				}
				else if(roleResult instanceof RoleUnresolved)
				{
					roleUnresList.add((RoleUnresolved)roleResult);
				}
			}
		}
		catch(IllegalArgumentException e)
		{
			throw new RuntimeException(e.getMessage());
		}

		return new RoleResult(roleList,roleUnresList);
    }

    /**
     *	This method sets the value of a Role.
     */
    Object setRoleFriend(Role theRole,boolean theMultiRoleFlg,
    	RelationService service)
			throws IllegalArgumentException,RoleNotFoundException,InvalidRoleValueException,RelationTypeNotFoundException,RelationNotFoundException,RelationServiceNotRegisteredException
	{
		if(theRole == null)
		{
			throw new IllegalArgumentException();
		}

		//It is very similar to getRoleFriend method implementation.
		String roleName = theRole.getRoleName();
		int problemType = 0;

		Role role = (Role)(roleToValueTable.get(roleName));

		ArrayList originalValue = null;
		Boolean initFlag = null;
		if(role == null)
		{
			originalValue = new ArrayList();
			initFlag = new Boolean(true);
		}
		else
		{
			initFlag = new Boolean(false);
			originalValue = (ArrayList)(role.getRoleValue());
		}

		//check role writing before updating the table.
		try
		{
			if(service != null)
			{
				problemType =
				(service.checkRoleWriting(theRole,relTypeName,initFlag)).intValue();
			}
			else
			{
				Object[] params = new Object[]{theRole,relTypeName,initFlag};
				String[] sig = new String[]{"javax.management.relation.Role","java.lang.String","java.lang.Boolean"};
				problemType = ((Integer)(mbeanServer.invoke(relServiceName,"checkRoleWriting",params,sig))).intValue();
			}
		}
		catch(MBeanException me)
		{
			if(me.getTargetException() instanceof RelationTypeNotFoundException)
			{
				throw (RelationTypeNotFoundException)(me.getTargetException());
			}
			else
			{
				throw new RuntimeException(me.getMessage());
			}
		}
		catch(InstanceNotFoundException ie)
		{
			throw new RelationServiceNotRegisteredException(ie.getMessage());
		}
		catch(Exception e)
		{
			throw new RuntimeException(e.getMessage());
		}


		Object toRet = null;

		if(problemType == 0)
		{
			//Means role can be set.
			//Before updating the table send a role updateNotification
			if(! initFlag.booleanValue() )
	 		{
				Object[] params = new Object[]{relId,theRole,originalValue};
				String[] sig = new String[]{"java.lang.String","javax.management.relation.Role","java.util.List"};

				try
				{
					if(service != null)
					{
						service.sendRoleUpdateNotification(relId,theRole,originalValue);
						service.updateRoleMap(relId,theRole,originalValue);
					}
					else
					{
						mbeanServer.invoke(relServiceName,
									"sendRoleUpdateNotification", params, sig);

						//update the roleMap
						mbeanServer.invoke(relServiceName,"updateRoleMap",
															params, sig);
					}
				}
				catch(InstanceNotFoundException ie)
				{
					throw new RelationServiceNotRegisteredException((ie.getMessage()));
				}
				catch(MBeanException me)
				{
					if(me.getTargetException()instanceof RelationNotFoundException)
					{
						throw ((RelationNotFoundException)(me.getTargetException()));
					}
					else
					{
						throw new RuntimeException(me.getMessage());
					}
				}
				catch(Exception e)
				{
					throw new RuntimeException(e.getMessage());
				}
			}
			roleToValueTable.put(new String(roleName),(Role)(theRole.clone()));
			if(theMultiRoleFlg)
			{
				toRet = theRole;
			}
		}
		else
		{
			//Role cannot be set.
			if (!(theMultiRoleFlg))
			{
				//try
				//{
					if(problemType == RoleStatus.NO_ROLE_WITH_NAME ||
					problemType == RoleStatus.ROLE_NOT_READABLE ||
					problemType == RoleStatus.ROLE_NOT_WRITABLE)
					{
						throw new RoleNotFoundException();
					}
					else
					{
						throw new InvalidRoleValueException();
					}
				//}catch(InvalidRoleValueException ie)
				//{
				//	throw new RuntimeException(ie.getMessage());
				//}
			}
			else
			{
				toRet = new RoleUnresolved(roleName,theRole.getRoleValue(),
											problemType);
			}
		}
		return toRet;
	}
}