/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Vector;

/**
 * A RelationTypeSupport object implements the RelationType interface.
 * It represents a relation type, providing role information for each role
 * expected to be supported in every relation of that type.
 *
 * A relation type includes a relation type name and a list of role infos
 * (represented by RoleInfo objects).
 *
 * A relation type has to be declared in the Relation Service:
 * 				- either using the createRelationType() method, where a
 * 					RelationTypeSupport object will be created
 * 					and kept in the Relation Service
 * 				- either using the addRelationType() method where the user has
 * 					to create an object implementing the RelationType interface,
 * 					and this object will be used as representing a relation
 * 					type in the Relation Service.
 */
public class RelationTypeSupport implements RelationType
{
	String relTypeName = null;

	RoleInfo [] roleInfoArray = null;

	Vector roleInfo = new Vector();

	private String ERR_VALUE = "";

    private HashMap myRoleName2InfoMap = new HashMap();


    /**
	 * Constructs a RelationTypeSupport with the relation type name specified.
	 *
	 * @param theRelTypeName The string argument specifying the name
	 * 				for RelationTypeSupport
	 */
	protected RelationTypeSupport(String theRelTypeName)
	{
		this.relTypeName = theRelTypeName;
	}

	/**
	 * Constructor where all role definitions are dynamically created and
	 * passed as parameter.
	 *
	 * @param  theRelTypeName - Name of relation type
	 *
	 * @param  theRoleInfoArray - List of role definitions (RoleInfo objects)
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 *
	 * @exception InvalidRelationTypeException - if:
	 * 				- the same name has been used for two different roles
	 *  			- no role info provided
	 *  			- one null role info provided
	 */
	public RelationTypeSupport(String theRelTypeName, RoleInfo[] theRoleInfoArray)
                    	throws IllegalArgumentException,
	            		   	   InvalidRelationTypeException
	{
		if(theRelTypeName == null || theRoleInfoArray == null)
			throw new IllegalArgumentException(
					"Cannot Create a RelationTypeSupport with null values!!!!!");

		this.relTypeName = theRelTypeName;
		this.roleInfoArray = theRoleInfoArray;

		checkRoleInfoConsistency(theRoleInfoArray);

		for(int i = 0; i < theRoleInfoArray.length; i++)
		{
		    RoleInfo currRoleInfo = theRoleInfoArray[i];
		    myRoleName2InfoMap.put(new String(currRoleInfo.getName()),
				   new RoleInfo(currRoleInfo));
		}
	}

	protected void addRoleInfo(RoleInfo theRoleInfo)
                    throws IllegalArgumentException,
	            		   InvalidRelationTypeException
	{
		if(theRoleInfo == null)
		    throw new IllegalArgumentException();

		if(	myRoleName2InfoMap.containsKey(theRoleInfo.getName()))
		{
			throw new InvalidRelationTypeException("RoleInfo already present");
		}
		else
		{
			myRoleName2InfoMap.put(theRoleInfo.getName(),new RoleInfo(theRoleInfo));
		}
	}

	/**
	 * Returns the relation type name.
	 *
	 * Specified by:
	 *           getRelationTypeName in interface RelationType
	 *
	 * @return The frelation type name
	 */
	public String getRelationTypeName()
	{
		return this.relTypeName;
	}

	/**
	 * Returns the role info (RoleInfo object) for the given role info name
	 * (null if not found).
	 *
	 * Specified by:
	 *       getRoleInfo in interface RelationType
	 *
	 * @param theRoleInfoName - role info name
	 *
	 * @return RoleInfo object providing role definition does not exist
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 *
	 * @exception RoleInfoNotFoundException - if no role info with that
	 * 				name in relation type.
	 */
	public RoleInfo getRoleInfo(String theRoleInfoName)
	                     throws IllegalArgumentException,
								RoleInfoNotFoundException
	{
		if( theRoleInfoName == null)
			throw new IllegalArgumentException();

	    RoleInfo toRet = getRoleInfoFromMap(theRoleInfoName);

	    if(toRet == null)
	        throw new RoleInfoNotFoundException();

		return toRet;
	}

	/**
	 * Returns the list of role definitions (ArrayList of RoleInfo objects).
	 *
	 * Specified by
	 *   getRoleInfos in interface RelationType
	 *
	 * @return The list of role definitions
	 */
	public List getRoleInfos()
	{
		//put all the elements of the RoleInfoArray into a ArayList and return it.
		ArrayList toRet = new ArrayList(myRoleName2InfoMap.values());

		return toRet;
	}


	// ************************** private methods ********************************* //

	// This method validates the RoleInfo array to contain any null elements
	// or if the array itself is null or any duplicate RoleInfos.
	private boolean validateRoleInfoArray(RoleInfo [] info)
	{
		if(info == null)
		{
			ERR_VALUE = "Cannot create RelationType for null RoleInfo";
			return false;
		}

		Hashtable temp = new Hashtable();

		for(int i=0;i<info.length;i++)
		{
			if(info[i] == null)
			{
				ERR_VALUE="Cannot create RelationType for null RoleInfo at"+ i +" st Index";
				return false;
			}
			else
			{
				if(! (temp.containsKey( ((RoleInfo)info[i]).getName() ) ) )
				{
					temp.put( ((RoleInfo)info[i]).getName(),new Object() );
				}
				else
				{
					ERR_VALUE="Cannot create RelationType duplicate RoleInfos";
					return false;
				}
			}
		}

		return true;
	}

	private  void checkRoleInfoConsistency(RoleInfo[] theRoleInfoArray)
							throws InvalidRelationTypeException
	{
		if(!( theRoleInfoArray.length >0))
					throw new InvalidRelationTypeException("No elements in the RoleInfo");

		if(! ( this.validateRoleInfoArray(theRoleInfoArray) ) )
			throw new InvalidRelationTypeException("ERR_VALUE");
	}

	private RoleInfo getRoleInfoFromMap(String roleName)
	{
		return (RoleInfo)(myRoleName2InfoMap.get(roleName));
	}

	//This method gets the RoleInfo from the Array given the RoleInfo name.
	private RoleInfo getRoleInfoFromVector(String roleInfoName)
	{
        RoleInfo toRet = null;

		for(int i=0;i<this.roleInfo.size();i++)
		{
			if(roleInfoName.equals(((RoleInfo)(roleInfo.elementAt(i))).getName()))
			{
				toRet = (RoleInfo)(roleInfo.elementAt(i));
				break;
			}
		}

	    return toRet;
	}
}